# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import traceback
import configparser
from ..ml_utils import pre_utils
from ..systemlog import sysmanagerlog, syserrorlog
from ..ml_utils import runstatus


class PreFeatureUtils(object):
    """
    Data Feature preprocessing

    Parameters
    --------
    data : pandas.DataFrame
    conf : configparser.ConfigParser

    Examples
    --------
    >>> from xes_ml_arch.src.feature_enginnering import pre_feature_utils
    >>> import configparser
    >>> import pandas as pd
    >>> conf = configparser.ConfigParser()
    >>> conf.read("pre_feature.conf")
    >>> data = pd.read_csv("my_data.csv")
    # this
    >>> ins = pre_feature_utils.PreFeatureUtils(data=data, conf=conf)
    >>> ins.init()
    >>> ins.excute()
    # or this
    >>> ins = pre_feature_utils.PreFeatureUtils(conf=conf)
    >>> ins.init()
    >>> ins.set_data(data)
    >>> ins.excute()

    pre_feature.conf
    ----------------
    [pre_feature_utils]
    pre_flag = True    # if use Feature preprocessing or not
    single_feature_apply = {"col0":"time2stamp", "col1":"stamp2time"}
    multi_feature_apply = {'col0-col1':'minus_data', 'col2-col3':'abs_minus_data'}
    # which_feature_apply = {col_name: function name in utils}
    """

    FEATURE_PROC = "feature_processor.FeatureProcessor."
    PRE_FEATURE = 'pre_feature_utils'
    PRE_FLAG = 'pre_flag'
    SINGLE_APP = 'single_feature_apply'
    MULTI_APP = 'multi_feature_apply'
    DASH = ','

    def __init__(self, data=None, conf=None, log_path = None):
        self._conf = conf
        self.xeasy_log_path = log_path
        self._data = data
        self._single_app = {}
        self._multi_app = {}
        self._feature_function = ''
        self._pre_flag = False
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__,self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.xeasy_log_path)

    def init(self):
        """
        Init feature pre utils object.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        try:
            if not isinstance(self._conf, configparser.ConfigParser):
                self.managerlogger.logger.error("init pre feature_utils error: config is illegal")
                return runstatus.RunStatus.FAILED

            self._pre_flag = eval(self._conf.get(self.PRE_FEATURE, self.PRE_FLAG))
            if not self._pre_flag:
                return runstatus.RunStatus.SUCC
            if self.SINGLE_APP in self._conf.options(self.PRE_FEATURE):
                self._single_app = eval(self._conf.get(self.PRE_FEATURE, self.SINGLE_APP))
            if self.MULTI_APP in self._conf.options(self.PRE_FEATURE):
                self._multi_app = eval(self._conf.get(self.PRE_FEATURE, self.MULTI_APP))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("feature pre process init error: %s" % err)
            self.errorlogger.logger.error("feature pre process init error:\n %s" % traceback.format_exc())
        return runstatus.RunStatus.FAILED

    def set_data(self, data):
        """
        Set data.

        Parameters
        --------
        data: pandas.DataFrame
            data to feature pre,

        Returns
        --------
        :return: bool
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: False
        """
        if not pre_utils.PredictUtils.valid_pandas_data(data):
            self.managerlogger.logger.error("feature pre process set data error")
            return runstatus.RunStatus.FAILED
        self._data = data
        return runstatus.RunStatus.SUCC

    def excute(self):
        '''
        Excute.

        Returns
        --------
        self._data: processed data
        '''
        if not self._init():
            return None
        if not self._pre_flag:
            self.managerlogger.logger.info("no need pre feature processer")
            return self._data
        return self.pre_feature_process()

    def _init(self):
        """
        Here, we convert all column names into lowercase letters, confirm that the data
        is a pandas DataFrame and the length is not zero.

        Returns
        ------
        :return : bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        if not pre_utils.PredictUtils.valid_pandas_data(self._data):
            self.errorlogger.logger.error("pre feature data is not DataFram: %s" % type(self._data))
            return runstatus.RunStatus.FAILED
        try:
            # Convert all column names to lowercase letters.
            self._data = self._data.copy()
            self._data.rename(columns=lambda x: x.lower(), inplace=True)
            return runstatus.RunStatus.SUCC
        except:
            self.errorlogger.logger.error(traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def _single_feature_process(self):
        """
        One column of data feature processor.

        Returns
        ------
        :return : bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        try:
            for key in self._single_app:
                key_new = key.replace(' ', '')
                _data_in = self._data[key_new.lower()]
                _fun = self._single_app[key_new]
                _res = eval(self.FEATURE_PROC + _fun)(_data_in)
                if _res is not None:
                    self._data[key_new.lower()] = _res
                else:
                    self.managerlogger.logger.error(eval(self.FEATURE_PROC + _fun) + "error: %s " % key)
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("_signle_feature_process error: %s" % err)
            self.errorlogger.logger.error("_signle_feature_process error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def _multi_feature_process(self):
        """
        Multi-column data feature processor.

        Returns
        ------
        :return : bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        try:
            for key in self._multi_app:
                key_new = key.replace(' ', '')
                _col_list = key_new.lower().split(self.DASH)
                _fun = self._multi_app[key_new]
                if len(_col_list) == 2:
                    _data_in1 = self._data[_col_list[0]]
                    _data_in2 = self._data[_col_list[1]]
                    _res = eval(self.FEATURE_PROC + _fun)(_data_in1, _data_in2)
                    self._data[key_new.lower() + '_' + _fun] = _res
            return runstatus.RunStatus.SUCC
        except Exception as e:
            self.managerlogger.logger.error("_multi_feature_process error: %s" % e)
            self.errorlogger.logger.error("_multi_feature_process error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def pre_feature_process(self):
        """
        pre_feature_process

        Returns
        --------
        self._data: processed data
        """
        if self._single_app:
            self._single_feature_process()
        if self._multi_app:
            self._multi_feature_process()
        return self._data



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

# while unittest , look at the 370(line) of analysis.py
import os
import unittest
import sys
import configparser
import random
import pandas as pd
import numpy as np

sys.path.append("../../../..")

from xes_ml_arch.src.analysis import analysis


class TestAnalysis(unittest.TestCase):
    def setUp(self):
        self.conf = configparser.ConfigParser()
        self.conf_file = "analysis.conf"
        self.conf.read(self.conf_file)
        self.columns = ["col%s" % (x) for x in range(10)]
        self.x = self.columns[:9]
        self.y = self.columns[9]
        self.get_data()
        self.ins = analysis.Analysis(conf=self.conf, log_path = './config/log.conf' , data=self._data, feature_columns=self.x, label_columns=self.y)

    def test_execute(self):
        self.assertTrue(self.ins.execute())

    def test_reset(self):
        self.ins.reset(conf=self.conf, data=self._data, feature_columns=self.x,
                       label_columns=self.y)
        self.assertTrue(self.ins.execute())

    def get_data(self):
        self._data = pd.DataFrame(
            [[int(random.random() * 100) for x in range(10)] for y in range(10000)],
            columns=self.columns)
        self._data["col9"] = np.random.choice([0, 1], size=10000)

if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from .base_model import BaseModel
import time
import traceback
import numpy as np
from ..ml_utils import runstatus

try:
    import xgboost as xgb
except:
    pass


class MyXgb(BaseModel):
    """A xgboost classifier.

    Parameters
    -----------
    config: configparser.ConfigParser(), default = None
            Model configuration file.

    Examples
    -----------
       >>> from xes_ml_arch.src.model import my_xgb
       >>> import configparser
       >>> import pandas as pd
       >>> import numpy as np
       >>> import random
       >>> config = configparser.ConfigParser()
       >>> config.read("xgb.conf")
       >>> x = pd.read_csv('data.txt', sep=',')
       >>> y = np.array([int(random.random() * 100) for _x in range(99)])
       >>> ins = my_xgb.MyXgb(config=config)
       >>> ins.init()
       >>> ins.train(x, y)
       ...
       ...
    """

    XGB_N_ESTIMATORS = "num_boost_round"
    MODEL_ID = BaseModel.MODEL_XGB

    def __init__(self, config=None, log_path = None):
        super(MyXgb, self).__init__(config=config, log_path = log_path)
        self._xgb_param = [('eval_metric', 'auc'), ('eval_metric', 'ams@0')]

    def _init_model(self):
        """Init model"""

        self._init_config()
        self._model = None

    def _init_config(self):
        pass

    def train(self, x, y):
        """Model train func.

        Parameters
        ----------
        x: input data.
        y: label.

        Returns
        -------
        bool: True(train suceessed) or False(train faild)
        """

        try:
            t_start = time.time()
            self.managerlogger.logger.info("start xgboost..")
            train_data = xgb.DMatrix(x, label=y, missing=-999)
            num_round = 10
            if self.XGB_N_ESTIMATORS in self._xgb_param:
                num_round = self._xgb_param[self.XGB_N_ESTIMATORS]
                self._xgb_param.pop(self.XGB_N_ESTIMATORS)
            bst = xgb.train(self._model_params, train_data, num_round, [])
            t_end = time.time()
            self.managerlogger.logger.info("finished xgboost!")
            self.managerlogger.logger.info("xgboost train time: %s" % (t_end - t_start))

            self._model = bst
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("xgboost train error: %s " % err)
            self.errorlogger.logger.error("xgboost train error:\n %s " % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def predict(self, x, thresh=0.5):
        """Forecast input data.

        Parameters
        ----------
        x：input sample data.
        thresh: label classification threshold.

        Returns
        -------
        Prediction label(list).
        """

        if self._model is None:
            return None
        x_data = xgb.DMatrix(x)
        res = self._model.predict(x_data)
        res = [0 if x < thresh else 1 for x in res]
        return np.array(res)

    def predict_proba(self, x):
        """Calculate data predict probability value.

        Parameters
        ----------
        x: input sample data.

        Returns
        ---------
        list of probability value [[ ]]
        """

        if self._model is None:
            return None
        x_data = xgb.DMatrix(x)
        return self._model.predict(x_data)

    def get_feature_importance(self, feature):
        """Get feature weights of user data.

        Parameters
        ----------
        feature: Target characteristics(string of list).

        Returns
        ---------
        the score of feature importrance.(list, [[feature, score]]).
        """

        try:
            feature_importance = []
            importance = self._model.get_score(importance_type='gain')
            for key in importance:
                feature_importance.append([importance[key], key])
            return feature_importance
        except Exception as err:
            self.managerlogger.logger.error("xgboost get feature importance error: %s" % err)
            self.errorlogger.logger.error("xgboost get feature importance error:\n %s" % traceback.format_exc())
            return None



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import pandas as pd
import traceback
from ..systemlog import sysmanagerlog, syserrorlog
from ..systemlog import syserrorlog
from ..ml_utils import runstatus
from ..ml_utils import pre_utils


class DataSampler(object):
    """
    Data sampling of train data or test data.

    Parameters
    ---------
    conf: configparser.ConfigParser
        Configuration information
    data: pandas.DataFrame
    sample_rate: float
    target: col of data, label
    base_size:int, default = -1

    Examples
    --------
    >>> from xes_ml_arch.src.feature_enginnering import data_sampler
    >>> import configparser
    >>> import pandas as pd
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> data = pd.read_csv("my_data.csv")
    # this
    >>> ins = data_sampler.DataSampler(data=data, conf=conf, target='col9')
    >>> ins.init()
    >>> ins.excute()
    # or this
    >>> ins = data_sampler.DataSampler(conf=conf, target='col9')
    >>> ins.init()
    >>> ins.set_data(data=data)
    >>> ins.excute()
    """

    DATA = "data"
    VALUE = "value"
    SIZE = "size"
    RATE = "rate"
    DATA_SAMPLE = "data_sample"
    SAMPLE_RATE = "sample_rate"
    SAMPLE_FLAG = "sample_flag"

    def __init__(self, conf=None, log_path = None, data=None, sample_rate=None, target="", base_size=-1):
        self._conf = conf
        self.xeasy_log_path = log_path
        self._data = data
        self._sample_rate = sample_rate
        self._target = target
        self._base_size = base_size
        self._result_sample_data = None
        self._type_data = []
        self._sample_flag = True
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__,self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.xeasy_log_path)

    def init(self):
        """
        Init data sampler object
        Returns
        ---------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        try:
            self._sample_flag = eval(self._conf.get(self.DATA_SAMPLE, self.SAMPLE_FLAG))
            if not self._sample_flag:
                return runstatus.RunStatus.SUCC
            self._sample_rate = eval(self._conf.get(self.DATA_SAMPLE, self.SAMPLE_RATE))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("data sampler init error: %s" % err)
            self.errorlogger.logger.error("data sampler init error:\n %s" % traceback.format_exc())
        return runstatus.RunStatus.FAILED

    def set_target_field(self, target_str):
        """
        Set target.

        Parameters
        ----------
        target_str: str
            name of col, label

        Returns
        ----------
        :return: bool, runstatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        if not isinstance(target_str, str):
            return runstatus.RunStatus.FAILED
        self._target = target_str
        return runstatus.RunStatus.SUCC

    def excute(self):
        '''
        Data sampling start.

        Returns
        ----------
        :return: bool, runstatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        '''
        self.managerlogger.logger.info("start sample")
        # init
        if not self._sample_flag:
            self.managerlogger.logger.info("no need to sample")
            self._result_sample_data = self._data
            return runstatus.RunStatus.SUCC
        if self._init() == runstatus.RunStatus.FAILED:
            return runstatus.RunStatus.FAILED
        # sample
        if self._start_sample() == runstatus.RunStatus.FAILED:
            return runstatus.RunStatus.FAILED
        self.managerlogger.logger.info("finish sample")
        return runstatus.RunStatus.SUCC

    def get_data(self):
        """
        Get data.

        Returns
        ----------
        :return: result of data sample
        """
        return self._result_sample_data

    def set_data(self, data):
        """
        Set data.

        Parameters
        ----------
        data: pandas.DataFrame
            data need sample

        Returns
        ----------
        :return: bool, runstatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        if not pre_utils.PredictUtils.valid_pandas_data(data):
            return runstatus.RunStatus.FAILED
        self._data = data
        return runstatus.RunStatus.SUCC

    def _init(self):
        """
        Check input and init info of all class

        Returns
        ----------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """

        if not isinstance(self._data, pd.DataFrame):
            self.managerlogger.logger.error("input data is not dataframe")
            return runstatus.RunStatus.FAILED
        if self._target == "":
            self.managerlogger.logger.error("target not find")
            return runstatus.RunStatus.FAILED
        if self._sample_rate is None:
            self.managerlogger.logger.error("sample rate is required")
            return runstatus.RunStatus.FAILED
        return self._init_type_data()

    def _init_type_data(self):
        """
        Init info of all class:
        {data:samples, value:target value, size:numbers of sample, rate:rate of max(size)}

        Returns
        ----------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        # Get the number of samples for each category, and the label value.
        di_type_size = self._get_type_size()
        if len(self._sample_rate) != len(di_type_size):
            self.managerlogger.logger.error(
                "%s \n and %s \n not match" % (str(self._sample_rate), str(di_type_size)))
            return runstatus.RunStatus.FAILED
        try:
            for key in self._sample_rate:
                tmp_data = self._data[self._data[self._target] == key]
                tmp_size = di_type_size[key]
                tmp_rate = self._sample_rate[key]
                tmp_res_di = {self.DATA: tmp_data, self.SIZE: tmp_size, self.RATE: tmp_rate,
                              self.VALUE: key}
                self._type_data.append(tmp_res_di)
            self.managerlogger.logger.info("_init_type_data succeed")
            return runstatus.RunStatus.SUCC
        except Exception as e:
            self.managerlogger.logger.error("_init_type_data error: %s" % e)
            self.errorlogger.logger.error(traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def _get_type_size(self):
        """
        Get szie for every class

        Returns
        ---------
        :return: dict
            Return the name of each category and the corresponding quantity of each category.
        """
        count_series = self._data[self._target].value_counts()
        if self._base_size == -1:
            self._base_size = max(count_series.tolist())
        return dict(zip(count_series.index.tolist(), count_series.tolist()))

    def _start_sample(self):
        """
        Sample function.

        Returns
        ---------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        try:
            self._result_sample_data = pd.DataFrame(columns=self._data.columns)
            for di_value in self._type_data:
                sample_size = di_value[self.SIZE]
                sample_rate = di_value[self.RATE]
                sample_data = di_value[self.DATA]

                # Calculate the number of samples after sampling according to the relative
                # sampling ratio, compare with the actual number of samples, and select
                # over-sampling or under-sampling.
                target_size = int(self._base_size * sample_rate)
                if target_size < sample_size:
                    # under-sampling
                    sample_data = sample_data.sample(n=target_size, replace=False, random_state=37)
                elif target_size > sample_size:
                    # over-sampling
                    add_sample_data = sample_data.sample(n=target_size - sample_size, replace=True,
                                                         random_state=37)
                    sample_data = sample_data.append(add_sample_data)

                self._result_sample_data = self._result_sample_data.append(sample_data)
            # set type for all col
            for col in self._result_sample_data.columns:
                try:
                    self._result_sample_data[col] = self._result_sample_data[col].astype("float64")
                except:
                    self.errorlogger.logger.warning("%s can not astype float, which type is %s" % (
                    col, str(self._result_sample_data[col].dtype)))
            return runstatus.RunStatus.SUCC
        except Exception as e:
            self.managerlogger.logger.error("start sample error: %s" % e)
            self.errorlogger.logger.error("start sample error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import traceback
import pandas as pd
from ..systemlog import sysmanagerlog, syserrorlog
from ..ml_utils import runstatus
from ..ml_utils import pre_utils


class DataSplit(object):
    """
    This class is used to split a data set into train and test data set.

    Parameters
    --------
    conf: configparser.ConfigParser, default = None
        Configuration file for data set division.
    data: pandas.DataFrame, default = None
        Data set need split.

    Attributes
    --------
    _is_executed: bool, default = False
         The flag of the data set dividing.

    Examples
    --------
    >>> from xes_ml_arch.src.cross_validation import data_split
    >>> from xes_ml_arch.src.model import model_factory
    >>> import configparser
    >>> import pandas as pd
    >>> import numpy as np
    >>> import random
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> columns = ["col%s" % x for x in range(10)]
    >>> x = columns[:9]
    >>> y = columns[9]
    >>> data = pd.DataFrame(
    >>>           [[int(random.random() * 100) for _x in range(10)] for _y in range(1000)],
    >>>             columns=columns)
    >>> data["col9"] = np.random.choice([0, 1], size=1000)
    >>> ins = data_split.DataSplit(conf=conf, data=data)
    >>> ins.execute()
    >>> train_data = ins.train_data
    >>> test_data = ins.test_data
    >>> ins.store_train_data()
    >>> ins.store_test_data()
    """

    DATA_SPLIT = "data_split"
    RATIO = "test_ratio"
    TRAIN_FILE = "train_file"
    TEST_FILE = "test_file"
    LOAD_DATA_FROM_LOCAL_FILE = "load_data_from_local_file"
    TRUE = "true"

    def __init__(self, conf=None, log_path = None, data=None):
        self._conf = conf
        self.xeasy_log_path = log_path
        self._data = data
        self._train_data = None
        self._test_data = None
        self._is_executed = False

        self.managerlogger = sysmanagerlog.SysManagerLog(__file__, self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__, self.xeasy_log_path)

    def reset(self, conf=None, data=None):
        """
        Reset config and data.

        Parameters
        --------
        conf: configparser.ConfigParser, default = None
            Configuration file for data set division.
        data: pandas.DataFrame, default = None
            Data set need split.
        """
        if conf is not None:
            self._conf = conf
        if data is not None:
            self._data = data
        self._is_executed = False

    def execute(self):
        """
        Start dividing the data set.

        Returns
        --------
        Bool: runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        if self._is_executed:
            return runstatus.RunStatus.SUCC
        if not pre_utils.PredictUtils.valid_pandas_data(self._data):
            return runstatus.RunStatus.FAILED
        try:
            if self.LOAD_DATA_FROM_LOCAL_FILE in self._conf.options(self.DATA_SPLIT):
                if self._conf.get(self.DATA_SPLIT,
                                  self.LOAD_DATA_FROM_LOCAL_FILE).lower() == self.TRUE:
                    return self._load_data()
            self._shuf_data()
            # Dividing
            self._split_data()
            self._is_executed = True
            self.store_train_data()
            self.store_test_data()
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.errorlogger.logger.error("cross validation error\n" + traceback.format_exc())
            self.managerlogger.logger.error("cross validation error: %s" % err)
            return runstatus.RunStatus.FAILED

    @property
    def train_data(self):
        """
        Get train data as property.

        Examples
        --------
        usage: ds = data_split.DataSplit()
               train_data =ds.train_data

        Returns
        --------
        self._train_data : pandas.DatFrame
        """
        if pre_utils.PredictUtils.valid_pandas_data(self._train_data):
            return self._train_data
        else:
            raise TypeError("split train data error")

    @property
    def test_data(self):
        """
        Get test data as property.

        Examples
        --------
        usage: ds = data_split.DataSplit()
               test_data =ds.test_data

        Returns
        --------
        self._test_data : pandas.DateFrame
        """
        if pre_utils.PredictUtils.valid_pandas_data(self._test_data):
            return self._test_data
        else:
            raise TypeError("split test data error")

    def store_train_data(self):
        """Store the divided train data set to file."""

        return self._store_data(self.TRAIN_FILE, self._train_data)

    def store_test_data(self):
        """Store the divided test data set to file."""

        return self._store_data(self.TEST_FILE, self._test_data)

    def _split_data(self):
        """Divide orignal data into test data and train data."""
        # get sample ratio
        try:
            ratio = float(self._conf.get(self.DATA_SPLIT, self.RATIO))
        except TypeError:
            self.managerlogger.logger.info(
                "%s not found %s, use default value 0.1" % (self.DATA_SPLIT, self.RATIO))
            ratio = 0.1
        if ratio > 1 or ratio < 0:
            raise ValueError("sample ratio=%s, but range (0, 1) is allowed" % (ratio))
        cure = int(self._data.shape[0] * ratio)
        self._test_data = self._data.iloc[:cure]
        self._train_data = self._data.iloc[cure:]

    def _shuf_data(self):
        """Shuffle data."""
        # self._data = shuffle(self._data)
        self._data = self._data.sample(frac=1)

    def _store_data(self, result_conf, data):
        """
        Store data to file.

        Parameters
        --------
        result_conf: str
            File path in configuration file.
        data: pandas.DataFrame
            Data need to store.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        if not pre_utils.PredictUtils.valid_pandas_data(data):
            self.managerlogger.logger.error("train data is null")
            return runstatus.RunStatus.FAILED
        try:
            resutl_path = self._conf.get(self.DATA_SPLIT, result_conf)
        except Exception as err:
            self.managerlogger.logger.error("result path %s is not exist, please check out" % err)
            return runstatus.RunStatus.FAILED

        data.to_csv(resutl_path, index=False, encoding='utf8')
        return runstatus.RunStatus.SUCC

    def _load_data(self):
        """
        Load data from local file system.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        try:
            # get file path
            test_data_file = self._conf.get(self.DATA_SPLIT, self.TEST_FILE)
            train_data_file = self._conf.get(self.DATA_SPLIT, self.TRAIN_FILE)
            # read_file
            self._train_data = pd.read_csv(train_data_file)
            self._test_data = pd.read_csv(test_data_file)
            self._is_executed = True
            self.managerlogger.logger.info("load data success")
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("load error: %s" % err)
            self.errorlogger.logger.error("data split load data error! \n" + traceback.format_exc())
            return runstatus.RunStatus.FAILED



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from . import syslog


class SysErrorLog(syslog.SysLog):
    """Instantiate the system error log class"""
    def __init__(self, filename, log_path = None):
        syslog.SysLog.__init__(self, filename, name='error', path = log_path)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import configparser
import random

import pandas
import pandas as pd
import numpy as np

sys.path.append("../../../..")

from xes_ml_arch.src.cross_validation import cross_validation, data_split
from xes_ml_arch.src.model import base_model, model_factory


class TestCrossVaidation(unittest.TestCase):
    def setUp(self):
        self.conf = configparser.ConfigParser()
        self.conf_file = "cross.conf"
        self.conf.read(self.conf_file)
        self.columns = ["col%s" % (x) for x in range(10)]
        self.x = self.columns[:9]
        self.y = self.columns[9]
        self._data = pd.DataFrame(
            [[int(random.random() * 100) for x in range(10)] for y in range(10000)],
            columns=self.columns)
        self._data["col9"] = np.random.choice([0, 1], size=10000)

        self.create_config()
        self.ins = cross_validation.Cross_Validation(conf=self._cv_config, log_path = '../log/log.conf',data=self._data,
                                                     x_columns=self.x, y_column=self.y)
        self.ins_no = cross_validation.Cross_Validation(conf=self._cv_config, log_path = '../log/log.conf',data=pandas.DataFrame(),
                                                        x_columns=self.x, y_column=self.y)
        self.split_ins = data_split.DataSplit(conf=self._data_spilt_config, log_path = '../log/log.conf', data=self._data)

    def test_data_split(self):
        self.assertTrue(self.split_ins.execute())
        train_data = self.split_ins.train_data
        test_data = self.split_ins.test_data
        # assert (train_data.shape[0] == 9000)
        # assert (test_data.shape[0] == 1000)
        self.assertEqual(train_data.shape[0], 9000)
        self.assertEqual(test_data.shape[0], 1000)
        self.assertTrue(self.split_ins.store_test_data())
        self.assertTrue(self.split_ins.store_train_data())
        self.assertTrue(self.split_ins._load_data())
        self.assertFalse(self.split_ins._store_data(self._data_spilt_config, self._data))
        self.split_ins.reset(None, None)
        self.split_ins.reset(self._data_spilt_config, self._data)

    def test_data_split_no(self):
        self.split_ins_no = data_split.DataSplit(conf=self._data_spilt_config_no, log_path = '../log/log.conf',data=self._data)
        self.split_ins_no.reset()
        self.assertTrue(self.split_ins_no.execute())
        try:
            train_data = self.split_ins_no.train_data
            test_data = self.split_ins_no.test_data
            self.assertTrue(self.split_ins.store_test_data())
            self.assertTrue(self.split_ins.store_train_data())
        except:
            pass

    def test_cv_model(self):
        model = model_factory.ModelFactory.create_model(self._model_config,log_path = '../log/log.conf',
                                                        model_name= base_model.BaseModel.MODEL_SKLEARN_XGB)
        self.assertTrue(self.ins.execute())
        self.assertTrue(self.ins.cv_model(model))
        self.ins.reset(self._cv_config, self._data, 'col1', 'col2')

    def test_cv_model_1(self):
        model = None
        self.assertFalse(self.ins_no.execute())
        self.assertFalse(self.ins.cv_model(model))

    def create_config(self):
        self._model_config = configparser.ConfigParser()
        self._data_spilt_config = configparser.ConfigParser()
        self._cv_config = configparser.ConfigParser()
        self._data_spilt_config_no = configparser.ConfigParser()
        self._data_spilt_config.read("./data_split.conf")
        self._model_config.read("../../../config/demo/model_online.conf")
        self._cv_config.read("./cross_validation.conf")
        self._data_spilt_config_no.read("./data_split_try.conf")


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from ..systemlog import sysmanagerlog, syserrorlog
from . import data_washer
from . import feature_filter
from . import data_sampler
from . import xes_onehot_encoder
from . import pre_feature_utils
from ..ml_utils import runstatus
from ..ml_utils import global_pre
import configparser
import traceback


class DataProcessor(object):
    """
    Processor train data or test data.

    Parameters
    --------
    conf : configparser.ConfigParser, default = None
        Contains configuration information such as data set path, id field, target field, data processing method, etc.

    Attributes
    --------
    train_data : pandas.DataFrame, default = None
        The raw text data to learn. A 2D table, each row represents a sample, and each column represents
        the characteristics of a given sample.
    train_data_id : str, default = None
        The id field of train sample.
    train_data_feature : list, default = None
        The feature list of train sample.
    train_data_target : str, default = None
        The target field of train sample.

    test_data : pandas.DataFrame, default = None
        Test samples.
    test_data_id : str, default = None
        The id field of test sample.
    test_data_feature : list, default = None
        The feature list of test sample.
    test_data_target : str, default = None
        The target field of test sample.

    data_id_columns : list, default = None
        Id columns.
    data_feature_columns : list, default = None
        Feature columns.
    data_target_column : str, default = None
        Target column.
    _conf : configparser.ConfigParser, default = None
        Contains configuration information such as data set path, id field, target field, data processing
        method, etc.
    _feature_filter : :class : '~feature_filter.FeatureFilter'
        Feature selection, drop unused features and feature label.
    _data_sample : :class : '~data_sampler.DataSampler'
        Data sampling of train data or test data.
    _feature_washer : :class : '~data_washer.DataWasher'
        Sort column, zscore.
    _onehot_encoder : :class : '~xes_onehot_encoder.XESOneHotEncoder'
        Feature selection, drop unused features and feature label.

    Examples
    --------
    >>> from xes_ml_arch.src.feature_enginnering import data_processor
    >>> import configparser
    >>> import pandas as pd
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> feature_processor = data_processor.DataProcessor(conf=conf)
    >>> feature_processor.init()
    >>> train_data = pd.read_csv("my_traindata.csv")
    >>> test_data = pd.read_csv("my_testdata.csv")
    >>> feature_processor.train_data = train_data
    >>> feature_processor.test_data = test_data
    >>> feature_processor.execute()
    """
    # main config path
    default_config_file = "../config/demo/feature_enginnering.conf"
    # test config path
    # default_config_file = "./conf/feature_enginnering.conf"
    BASE_CONFIG = "base_config"
    ID_FIELDS = "id_fields"
    TARGET_FIELDS = "target_fields"
    TRAIN_DATA_FILE = "train_data_file"
    TEST_DATA_FILE = "test_data_file"

    def __init__(self, conf=None,log_path = None):

        self.xeasy_log_path = log_path
        # train data, 3 parts: id, feature, target
        self.train_data = None
        self.train_data_id = None
        self.train_data_feature = None
        self.train_data_target = None

        # test data, 3 parts: id, feature, target
        self.test_data = None
        self.test_data_id = None
        self.test_data_feature = None
        self.test_data_target = None

        # id columns, feature columns, target column
        self.data_id_columns = None
        self.data_feature_columns = None
        self.data_target_column = None

        # config
        self._conf = conf

        # data processor object
        self._feature_filter = None
        self._data_sample = None
        self._feature_washer = None
        self._onehot_encoder = None

        self.managerlogger= sysmanagerlog.SysManagerLog(__file__,self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.xeasy_log_path)

    def init(self):
        """
        Init feature processor object, including data sampling, data washing, feature preprocessing, etc.

        Returns
        --------
        :return : bool , runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        Notes
        --------
        If there is no usable conf parameter, read class default_config_file.
        """
        if not isinstance(self._conf, configparser.ConfigParser):
            self.managerlogger.logger.error("conf error: conf is not ConfigParser instance")
            config = configparser.ConfigParser()
            config.read(self.default_config_file)
            self._conf = config
        try:
            self.data_id_columns = self._conf.get(self.BASE_CONFIG, self.ID_FIELDS).lower().split(
                global_pre.Global.COMMA)
            self.data_target_column = self._conf.get(self.BASE_CONFIG, self.TARGET_FIELDS).lower()

            if not self._init_data_sample() or not self._init_data_washer() or \
                    not self._init_feature_filter() or not self._init_pre_feature() or \
                    not self._init_onehot_encoder():
                self.managerlogger.logger.error("feature processor init error")
                return runstatus.RunStatus.FAILED

            return runstatus.RunStatus.SUCC
        except Exception as ex:
            self.managerlogger.logger.error("data processor object init erorr: %s" % ex)
            self.errorlogger.logger.error("data processor object init erorr \n" + traceback.format_exc())
        return runstatus.RunStatus.FAILED

    """
    # set_data no use
    def set_data(self, data):
        # :param data:
        # :return: runstatus.RunStatus
        if not pre_utils.PredictUtils.valid_pandas_data(data):
            self.managerlogger.logger.logger.info("data is illegal")
            return runstatus.RunStatus.FAILED
        self._data = data
        return runstatus.RunStatus.SUCC
    """

    def execute(self):
        """
        Start data processor.

        Returns
        --------
        :return: bool, runstatus.RunStatus
             runstatus.RunStatus.SUCC: True
             runstatus.RunStatus.FAILED: Failed
        """
        if self.train_data is None and (self.test_data is not None and self._test_data_handle()) \
                or (self.train_data is not None and self._train_data_handle() and
                    self.test_data is not None and self._test_data_handle()):
            self.managerlogger.logger.info("data processor succeed! ")
            return runstatus.RunStatus.SUCC
        else:
            self.managerlogger.logger.error("data processor error! ")
            return runstatus.RunStatus.FAILED

    def _init_pre_feature(self):
        # init class object PreFeatureUtils.
        self._feature_preprocesser = pre_feature_utils.PreFeatureUtils(conf=self._conf, log_path = self.xeasy_log_path)
        if self._feature_preprocesser.init() == runstatus.RunStatus.SUCC:
            return True
        else:
            self.managerlogger.logger.error("init feature preprocesser error")
            return False

    def _init_data_washer(self):
        # init class object DataWasher.
        self._feature_washer = data_washer.DataWasher(conf=self._conf, log_path = self.xeasy_log_path)
        if self._feature_washer.init() == runstatus.RunStatus.SUCC:
            return True
        else:
            self.managerlogger.logger.error("init data washer error")
            return False

    def _init_feature_filter(self):
        # init class object FeatureFilter.
        self._feature_filter = feature_filter.FeatureFilter(conf=self._conf, log_path = self.xeasy_log_path)
        if self._feature_filter.init() == runstatus.RunStatus.SUCC:
            return True
        else:
            self.managerlogger.logger.error("init feature filter error")
            return False

    def _init_data_sample(self):
        # init class object DataSampler.
        self._data_sample = data_sampler.DataSampler(conf=self._conf,
                                                     log_path = self.xeasy_log_path,
                                                     target=self.data_target_column)
        if self._data_sample.init() == runstatus.RunStatus.SUCC:
            return True
        else:
            self.managerlogger.logger.error("init data sample error")
            return False

    def _init_onehot_encoder(self):
        # init class object XESOneHotEncoder.
        self._onehot_encoder = xes_onehot_encoder.XESOneHotEncoder(None, conf=self._conf,log_path = self.xeasy_log_path)
        if self._onehot_encoder.init() == runstatus.RunStatus.SUCC:
            return True
        else:
            self.managerlogger.logger.error("init data sample error")
            return False

    def _train_data_handle(self):
        """
        Handle train data. Including feature preprocessing, data filtering, converting features
        into one hot encoding, data sampling and data washer.
        After these processes, we can get the data for training

        Returns
        --------
        :return: bool
            True: succeed
            False: failed
        """
        try:
            # feature preprocessing
            if self._feature_preprocesser.set_data(self.train_data) == runstatus.RunStatus.FAILED:
                return False
            self.train_data = self._feature_preprocesser.excute()

            # filter data
            if self._feature_filter.set_data(self.train_data) == runstatus.RunStatus.FAILED:
                return False
            self.train_data = self._feature_filter.excute()
            if self.train_data is None:
                self.managerlogger.logger.error("data filter error")
                return False

            # Converting features into one hot encoding.
            self._onehot_encoder.set_data(self.train_data)
            if self._onehot_encoder.execute():
                self.train_data = self._onehot_encoder.get_data()

            # store processed data
            self._store_data(self.TRAIN_DATA_FILE, self.train_data)

            # sample data
            if self._data_sample.set_data(self.train_data) == runstatus.RunStatus.FAILED:
                return False
            if self._data_sample.excute() == runstatus.RunStatus.FAILED:
                self.managerlogger.logger.error("data sample faild")
                return False

            # split train data to 3 parts
            self.train_data = self._data_sample.get_data()
            self.train_data_id = self.train_data[self.data_id_columns]
            self.train_data_target = self.train_data[self.data_target_column]
            self.train_data_feature = self.train_data.drop(self.data_id_columns, 1).drop(
                self.data_target_column, 1)

            # data washer
            self._feature_washer.set_train_flag()
            self._feature_washer.set_data(self.train_data_feature)
            self.train_data_feature = self._feature_washer.excute()

            # get train data columns
            self.data_feature_columns = self.train_data_feature.columns
            if self.train_data_feature is not None:
                return True

        except Exception as err:
            self.managerlogger.logger.error("train data processor train data handle error: %s" % err)
            self.errorlogger.logger.error("_train_data_handle error: %s" % traceback.format_exc())

    def _test_data_handle(self):
        """
        Handle test data
        Returns
        --------
        :return: bool
            True : success
            False : faild
        """
        try:
            # feature preprocessing
            if self._feature_preprocesser.set_data(self.test_data) == runstatus.RunStatus.FAILED:
                return False
            self.test_data = self._feature_preprocesser.excute()

            # filter data
            self._feature_filter.set_data(self.test_data)
            self.test_data = self._feature_filter.excute()

            # onehot
            self._onehot_encoder.set_data(self.test_data)
            if self._onehot_encoder.execute():
                self.test_data = self._onehot_encoder.get_data()

            # store processed data
            self._store_data(self.TEST_DATA_FILE, self.test_data)

            # split test data to 3 parts
            self.test_data_id = self.test_data[self.data_id_columns]
            self.test_data_feature = self.test_data.drop(self.data_id_columns, 1)
            if self.data_target_column in self.test_data.columns:
                self.test_data_feature = self.test_data_feature.drop(self.data_target_column, 1)
                self.test_data_target = self.test_data[self.data_target_column]

            # data washer
            self._feature_washer.set_test_flag()
            self._feature_washer.set_data(self.test_data_feature)
            self.test_data_feature = self._feature_washer.excute()

            # get test data columns
            self.data_feature_columns = self.test_data_feature.columns

            if self.test_data_feature is not None:
                return True
        except Exception as err:
            self.managerlogger.logger.error("test data processor data error: %s" % err)
            self.errorlogger.logger.error("_test_data_handle error: %s" % traceback.format_exc())
            return False

    def _store_data(self, optios, data):
        """
        Store processed data to file.
        Parameters
        ----------
        optios: str
            file to store, config options.
        data: pandas.DataFrame
            processed data.

        Returns
        ---------
        :return: bool
            True: succ
            False: failed
        """
        try:
            path = self._conf.get(self.BASE_CONFIG, optios)
            data.to_csv(path, index=False)
            return True
        except Exception as ex:
            self.managerlogger.logger.error("store data error: %s" % ex)
            self.errorlogger.logger.error("_store_data error:  %s " % traceback.format_exc())
        return False



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from .base_model import BaseModel
import time
import traceback
import numpy as np
from ..ml_utils import runstatus

try:
    import lightgbm as lgb
except:
    pass


class MyLightGBM(BaseModel):
    """A lightgbm classifier. The encapsulation form is base model.

    Parameters
    --------
    config: configparser.ConfigParser()
        Configuration file for model initialization.

    Examples
    --------
    >>> from xes_ml_arch.src.model import my_lightgbm
    >>> import configparser
    >>> import pandas as pd
    >>> import numpy as np
    >>> import random
    >>> config = configparser.ConfigParser()
    >>> config.read("lightgbm.conf")
    >>> x = pd.read_csv('data.txt', sep=',')
    >>> y = np.array([int(random.random() * 100) for _x in range(99)])
    >>> ins = my_lightgbm.MyLightGBM(config=config)
    >>> ins.init()
    >>> ins.train(x, y)
    ...
    ...
    """
    GBM_N_ESTIMATORS = "num_boost_round"

    MODEL_ID = BaseModel.MODEL_LIGHTGBM

    def __init__(self, config=None, log_path = None):
        super(MyLightGBM, self).__init__(config=config, log_path = log_path)
        self._lgb_param_ = []

    def _init_model(self):
        """Initialize model."""

        self._init_config()
        self._model = None

        self._model = lgb.LGBMClassifier(**self._model_params)

    def _init_config(self):
        pass

    def train(self, x, y):
        """Train lightgbm.

        Parameters
        --------
        x: pandas.DataFrame of shape (n_sample, n_features)
            sample data
        y: array-like of shape (n_samples,)
            Target vector relative to X.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            t_start = time.time()
            self.managerlogger.logger.info("start lightGBM..")
            self._model.fit(x, y)

            t_end = time.time()
            self.managerlogger.logger.info("finished lightGBM!")
            self.managerlogger.logger.info("lightGBM train time: %s" % (t_end - t_start))

            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("lightGBM train error: %s " % err)
            self.errorlogger.logger.error("lightGBM train error:\n %s " % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def predict(self, x):
        """Calculate data predict value.

        Parameters
        --------
        x: pandas.DataFrame with shape (n_samples, n_features)
            Features of test data.

        Returns
        --------
        res: array like of shape (n_sample, )
            Predict result.
        """
        if self._model is None:
            return None
        # x_data = lgb.Dataset(x)
        # Label classification threshold.
        thresh =0.5
        res = self._model.predict(x)
        res=[0 if x < thresh else 1 for x in res]
        return np.array(res)

    def predict_proba(self, x):
        """Calculate data predict probability value.

        Parameters
        --------
        x: pandas.DataFrame with shape (n_samples, n_features)
            Features of test data.

        Returns
        --------
        res: list
            List of probability value belonging to a certain class.
        """

        if self._model is None:
            return None
        # x_data = lgb.Dataset(x)
        res = self._model.predict(x)
        return res

    def get_feature_importance(self, feature):
        """Get weights of user data features.

        Returns
        --------
        res: list
            Feature importance, list like : [[feature, score]].
        """
        try:
            # feature_importance = []
            # importance = self._model.get_score(importance_type='gain')
            # for key in importance:
            #     feature_importance.append([importance[key], key])
            res = zip(self._model.booster_.feature_importance(importance_type='split'), self._model.booster_.feature_name())
            #res = zip(self._model.feature_importance(), feature)

            return res
        except Exception as err:
            self.managerlogger.logger.error("lightGBM get feature importance error: %s" % err)
            self.errorlogger.logger.error("lightGBM get feature importance error:\n %s" % traceback.format_exc())
            return None

    def fit(self,x,y):
        """Training with data set."""

        return  self.train(x,y)


# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import os
import logging
import logging.config

from ..ml_utils import runstatus
from ..ml_utils import jsonmanager


class SysLog():
    __info_dict = None

    def __init__(self, filename, name, path = None):
        """Initialization parameter information"""

        if not path:
            path = os.path.join(os.path.dirname(__file__),'../../config/log.conf')
        self.log_path = path
        logging.config.fileConfig(self.log_path, disable_existing_loggers=False)

        # test log config path
        # logging.config.fileConfig('../config/log.conf')
        self.name = name
        self.filename = filename
        self.logging = logging.getLogger(self.name)

        self.__info_dict = {}

    def __del__(self):
        if self.__info_dict is not None:
            self.__info_dict.clear()

    @property
    def logger(self):
        """According to self.name, find the corresponding log configuration in log.conf and return it"""
        return self.logging

    def info(self, message):
        """Log level: Info; Content write function.

        Parameters
        ----------
        message: Written documents(string).

        Returns
        -------
        Write message and file doctor current log level to the output log file.
        """
        self.logging.info('%s : %s' % (self.filename, message))

    def error(self, message):
        """Log level: error; Content write function.

        Parameters
        ----------
        message: Written documents(string).

        Returns
        -------
        Write message and file doctor current log level to the output log file.
        """

        self.logging.error('%s : %s' % (self.filename, message))

    def debug(self, message):
        """Log level: debug; Content write function.

        Parameters
        ----------
        message: Written documents(string).

        Returns
        -------
        Write message and file doctor current log level to the output log file.
        """
        self.logging.debug('%s : %s' % (self.filename, message))

    def warning(self, message):
        """Log level: error; Content write function.

        Parameters
        ----------
        message: Written documents(string).

        Returns
         -------
        Write message and file doctor current log level to the output log file.
        """
        self.logging.warning('%s : %s' % (self.filename, message))

    def push_info(self, info_key, info_value):
        """Judge whether the log information is repeated.

        Parameters
        ----------
        info_key：log level.
        info_value: message.

        Returns
        -------
        If it already exists, return False; Otherwise recorded in the self.__info_dict.
        """
        if info_key in self.__info_dict:
            self.warning("key[%s] already exist in log" % (info_key))
            return runstatus.RunStatus.FAILED

        self.__info_dict[info_key] = info_value

    def print_info(self):
        """Output the log under the current file to the console"""

        self.logging.info(str(self.filename) + ':' + str(self.__info_dict))

    def object_info(self, message):
        """Convert Python data structure to JSON data structure and write it to log file; log level: info"""
        json_str = jsonmanager.get_message(message)
        self.info(json_str)

    def object_debug(self, message):
        """Convert Python data structure to JSON data structure and write it to log file; log level: debug"""
        json_str = jsonmanager.get_message(message)
        self.debug(json_str)

    def object_warning(self, message):
        """Convert Python data structure to JSON data structure and write it to log file; log level: warning"""
        json_str = jsonmanager.get_message(message)
        self.warning(json_str)

    def object_error(self, message):
        """Convert Python data structure to JSON data structure and write it to log file; log level: error"""
        json_str = jsonmanager.get_message(message)
        self.error(json_str)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import pandas as pd
import configparser
import traceback
import os
from ..analysis import analysis
from ..cross_validation import cross_validation, data_split
from ..feature_enginnering import data_processor
from ..systemlog import sysmanagerlog, syserrorlog
from ..constance.predict_constance import PredictConstance
from ..model import model_factory
from ..optimizing import optimizing
from ..ml_utils import runstatus, pre_utils, global_pre


class BaseML(object):
    """
    This basic class is a base framework for machine learning, used to solve a machine learning problem.

    Parameters
    --------
    conf : configparser.ConfigParser
        Model configuration file.

    Attributes
    ----------
    _data: pandas.DataFrame
        Machine learning data set.

    _conf: config object, instance of ConfigParser.ConfigParser

    _model: ml nodel object lr,rf,xgb...

    _predict_res: ml predict result

    _feature_processor: feature processor object

    _analysiser: ml analysiser

    _cross_validation: ml cross validation

    _data_spliter: data split object

    _result_path: the path to store ml result

    _log: system log

    _logerror: error debug log
    """

    def __init__(self, config=None,xeasy_log_path = None):
        # data
        self._data = None
        self.xeasy_log_path = xeasy_log_path

        # config  object
        self._conf = config

        # result path
        self._result_path = None

        # result
        self._predict_res = None

        # feature enginnering object
        self._feature_processor = None

        # model object
        self._model = None

        # analysis object
        self._analysiser = None

        # cross validataion object
        self._cross_validation = None

        # data spliter object
        self._data_spliter = data_split.DataSplit(log_path = self.xeasy_log_path)

        # optimizing
        self._optimizing = None
        # log object
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__,self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.xeasy_log_path)

    def init(self):
        """
        Initialize model,feature processer andanalysiser.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: success
            runstatus.RunStatus.FAILD: faild
        """
        try:
            self.managerlogger.logger.info("start ml...")
            if not isinstance(self._conf, configparser.ConfigParser):
                return runstatus.RunStatus.FAILED

            # store path for results
            self._result_path = global_pre.RES_PATH
            # unittest
            # self._result_path = '../result'

            if not os.path.isdir(self._result_path):
                os.mkdir(self._result_path)

            # init model, feature engineering and analysiser
            if self._init_model() and self._init_feature_processer()  and self._init_analysiser() and \
                    self._init_cv_ins() and self._init_optimizing():
                self.managerlogger.logger.info("ml init success")
                return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("machine_learning init error: %s" % err)
            self.errorlogger.logger.error("machine_learning init error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def start(self):
        pass

    def set_feature_processor_ins(self, instance):
        """
        Set feature processor object.

        Parameters
        --------
        instance: data_processor.DataProcessor
            Instance of data_processor.DataProcessor, to process data

        Returns
        --------
        :return: runstatus.RunStatus
            runstatus.RunStatus.SUCC: success
            return runstatus.RunStatus.FAILED: faild
        """
        if not isinstance(instance, data_processor.DataProcessor):
            self.managerlogger.logger.error("instance is not type of DataProcessor")
            return runstatus.RunStatus.FAILED
        self._feature_processor = instance
        return runstatus.RunStatus.SUCC

    def set_analysis_ins(self, instance):
        """
        Set analysiser.

        Parameters
        --------
        instance: analysis.Analysis
            Used to analysis a data set

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: success
            return runstatus.RunStatus.FAILED: faild
        """
        if not isinstance(instance, analysis.Analysis):
            self.managerlogger.logger.error("instance is not type of analysis.Analysis")
            return runstatus.RunStatus.FAILED
        self._analysiser = instance
        return runstatus.RunStatus.SUCC

    def set_optimizing_ins(self, instance):
        """
        Set the optimizer.

        Parameters
        --------
        instance: optimizing.Optimizing
            Used to search the best params.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: success
            return runstatus.RunStatus.FAILED: faild
        """
        if not isinstance(instance, optimizing.Optimizing):
            self.managerlogger.logger.error("instance is not type of optimizing.Optimizing")
            return runstatus.RunStatus.FAILED
        self._optimizing = instance
        return runstatus.RunStatus.SUCC

    def set_cross_validation(self, instance):
        """
        Set cross-validation object.

        Parameters
        --------
        instance: cross_validation.Cross_Validation
            Used to cross-validation.
        """
        if not isinstance(instance, cross_validation.Cross_Validation):
            self.managerlogger.logger.error("instance is not type of cross_validation.Cross_Validation")
        self._cross_validation = instance

    def set_data(self, data):
        """
        Set data.

        Parameters
        --------
        data: pandas.DataFrame
            Data set.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: success
            runstatus.RunStatus.FAILED: failed
        """
        if not isinstance(data, pd.DataFrame):
            return runstatus.RunStatus.FAILED
        self._data = data
        return runstatus.RunStatus.SUCC

    def _init_model(self):
        """
        Initialize model object.

        Returns
        --------
        :return: bool
            True: Success
            False: Faild
        """
        try:
            model_config = self._conf.get(PredictConstance.BASE_CONFIG,
                                          PredictConstance.MODEL_CONFIG)
            conf = configparser.ConfigParser()
            conf.read(model_config)
            self._model = model_factory.ModelFactory.create_model(config=conf)
            return True
        except Exception as err:
            self.managerlogger.logger.error("init model error: %s" % err)
            self.errorlogger.logger.error("init model error: \n %s" % traceback.format_exc())
            return False

    def _init_optimizing(self):
        """
        Initialize _init_optimizing object.

        Returns
        --------
        :return: bool
            True: Success
            False: Faild
        """
        try:
            model_config = self._conf.get(PredictConstance.BASE_CONFIG,
                                          PredictConstance.MODEL_CONFIG)
            conf = configparser.ConfigParser()
            conf.read(model_config)
            self._optimizing = optimizing.Optimizing(conf,log_path = self.xeasy_log_path)
            if self._optimizing.init() == runstatus.RunStatus.SUCC:
                return True
        except Exception as err:
            self.managerlogger.logger.error("init optimizing error: %s" % err)
            self.errorlogger.logger.error("init optimizing error: \n %s" % traceback.format_exc())
            return False


    def _init_feature_processer(self):
        """
        Initialize feature processer object.

        Returns
        --------
        :return: bool
            True: Success
            False: Faild
        """
        try:
            model_config = self._conf.get(PredictConstance.BASE_CONFIG,
                                          PredictConstance.FEATURE_ENGINEERING_CONFIG)
            conf = configparser.ConfigParser()
            conf.read(model_config)
            self._feature_processor = data_processor.DataProcessor(conf=conf,log_path = self.xeasy_log_path)
            if self._feature_processor.init() == runstatus.RunStatus.SUCC:
                return True
            else:
                return False
        except Exception as err:
            self.managerlogger.logger.error("init model error: %s" % err)
            self.errorlogger.logger.error("init model error:\n %s" % traceback.format_exc())
            return False

    def _init_analysiser(self):
        """
        Initialize analysiser.

        Returns
        --------
        :return: bool
            True: Success
            False: Faild

        """
        try:
            analysis_config = self._conf.get(PredictConstance.BASE_CONFIG,
                                             PredictConstance.ANALYSIS_CONFIG)
            conf = configparser.ConfigParser()
            conf.read(analysis_config)
        except Exception as err:
            self.managerlogger.logger.error("analysiser config error: %s" % err)
            return False
        self._analysiser = analysis.Analysis(conf=conf,log_path = self.xeasy_log_path)
        return True

    def _init_cv_ins(self):
        """
        Initialize cross validation.

        Returns
        --------
        :return: bool
            True: Success
            False: Faild
        """
        try:
            cross_val_conf = self._conf.get(PredictConstance.BASE_CONFIG,
                                            PredictConstance.CV_CONFIG)
            conf = configparser.ConfigParser()
            conf.read(cross_val_conf)
        except Exception as err:
            self.managerlogger.logger.error("init cv error: %s" % err)
            return False
        self._cross_validation = cross_validation.Cross_Validation(conf=conf,log_path = self.xeasy_log_path)
        return True

    def _get_result(self):
        """
        Get predict result.

        Returns
        --------
        :return: bool
            True: Success
            False: Faild
        """
        try:
            # get test data
            test_id = self._feature_processor.test_data_id
            test_feature = self._feature_processor.test_data_feature
            test_target = self._feature_processor.test_data_target

            # process data
            test_feature = test_feature.astype("float64", errors='ignore')

            # predict
            predict_res = self._model.predict(test_feature)
            predict_res_df = pd.DataFrame(predict_res, columns=[PredictConstance.PRE])
            proba_res = self._model.predict_proba(test_feature)
            proba_res_df = pd.DataFrame([str(x) for x in proba_res],
                                        columns=[PredictConstance.PROBA])

            res = [test_id, predict_res_df, proba_res_df]
            # get model score
            if test_target is not None:
                res.append(test_target)
                model_auc = pre_utils.PredictUtils.get_roc_score(test_target, proba_res)
                model_score = pre_utils.PredictUtils.get_model_score(test_target, predict_res)
                model_score.update(model_auc)
                with open(os.path.join(self._result_path, PredictConstance.TEST_SCORE), "w") as ftp:
                    ftp.write(str(model_score))

            # joint predict result
            self._joint_predict_result(res)

            return True
        except Exception as err:
            self.managerlogger.logger.error("base ml get result error: %s" % err)
            self.errorlogger.logger.error("base ml get result error:\n %s" % traceback.format_exc())
            return False

    def _joint_predict_result(self, result):
        """
        Joint the result.

        Parameters
        --------
        result: list(pandas.Dataframe)
            Results of prediction.

        Returns
        --------
        :return: bool
            True: Suceess
            False: Faild
        """
        if len(result) == 0:
            return
        try:
            [x.reset_index(drop=True, inplace=True) for x in result]
            self._predict_res = pd.concat(result, axis=1)
            return True
        except Exception as err:
            self.managerlogger.logger.error("joint_predict_result error: %s" % err)
            self.errorlogger.logger.error("joint_predict_result error:\n %s" % traceback.format_exc())
            return False

    def _store_predict_result(self):
        """
        Store result to file.

        Returns
        --------
        :return: bool
            True: Suceess
            False: Faild
        """
        try:
            self._predict_res.to_csv(os.path.join(self._result_path, PredictConstance.PREDICT_FILE),
                                     index=False)
            return True
        except Exception as err:
            self.managerlogger.logger.error("joint_predict_result error: %s" % err)
            self.errorlogger.logger.error("joint_predict_result error:\n %s" % traceback.format_exc())

            return False

    def _train_handle(self, data):
        """
        Train a model.

        Parameters
        --------
        data: pandas.DataFrame
            Input data.
        """
        pass

    def _predict_handle(self):
        """
        Predict handle.
        """
        pass



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import traceback
import os
import pandas as pd
from ..systemlog import sysmanagerlog, syserrorlog
from ..ml_utils import runstatus
from ..model import base_model
from ..ml_utils import pre_utils, global_pre
from ..constance.predict_constance import PredictConstance


class Cross_Validation(object):
    """
    Cross-validation class, and the methods provided include: training set test set division,
    cross training set test set division, model cross-validation effect.
.
    Parameters
    --------
    conf : configparser.ConfigParser, default = None
        Configuration information.
    data : pandas.DataFrame, default = None
        Data used for cross-validation.
    x_columns : List, default = None
        Name list of feature fields.
    y_column : str, default = ""
        Name of label field.
    id_fields : list, default = []
        Name list of id_fields.

    Attributes
    --------
    _k_fold : int, default = 7
        The value of k-fold cross-validation. Generally speaking, if the training data set is relatively small, increase the value of k. If the
        training set is relatively large, decrease the value of k.
    _is_executed : bool, default = False
        Cross-validation flag.

    Examples
    --------
    >>> from xes_ml_arch.src.cross_validation import cross_validation
    >>> from xes_ml_arch.src.model import model_factory
    >>> import configparser
    >>> import pandas as pd
    >>> import numpy as np
    >>> import random
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> model_conf = configparser.ConfigParser()
    >>> model_conf.read("model_config.conf")
    >>> model_name = 'sklearn_xgb'
    >>> columns = ["col%s" % x for x in range(10)]
    >>> x = columns[:9]
    >>> y = columns[9]
    >>> data = pd.DataFrame(
    >>>           [[int(random.random() * 100) for _x in range(10)] for _y in range(1000)],
    >>>            columns=columns)
    >>> data["col9"] = np.random.choice([0, 1], size=1000)
    >>> model = model_factory.ModelFactory.create_model(model_conf, model_name=model_name)
    # this
    >>> ins = cross_validation.Cross_Validation(conf=conf, data=data, x_columns=x, y_column=y)
    >>> ins.execute()
    >>> ins.cv_model(model)
    # or this
    >>> ins = cross_validation.Cross_Validation(conf=conf)
    >>> ins.reset(data=data,x_columns=x,y_column=y)
    >>> ins.execute()
    >>> ins.cv_model(model)
    """

    CROSS_VALIDATION = "cross_validation"
    RATIO = "train_test_tatio"
    K_FOLD = "k_fold"
    RESULT_FILE = "result_file"
    TRAIN_FILE = "train_file"
    TEST_FILE = "test_file"
    PREDICT_FILE = "predict_file"
    VERIFICATION_MODE = 'mode'
    IS_SHUFFLE = 'is_shuffle'
    GROUP = 'group'
    SEED = 'seed'
    VALID = 'valid'

    @staticmethod
    def _valid_data(data):
        """
        Static function to check data.

        Returns
        --------
        :return: bool
            True: succeed
            False: failed
        """
        if data is None:
            return False
        if not isinstance(data, pd.DataFrame):
            return False
        if data.shape[0] == 0:
            return False
        return True

    def __init__(self, conf=None,log_path = None, data=None, x_columns=None, y_column="", id_fields=list()):
        self._conf = conf
        self._data = data
        self.xeasy_log_path = log_path
        self._x_columns = x_columns
        self._y_column = y_column

        self._cv_data_list = []
        # k fold default:7
        self._k_fold = 7

        self._is_executed = False

        self._id_fields = id_fields

        self.managerlogger = sysmanagerlog.SysManagerLog(__file__,self.xeasy_log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,self.xeasy_log_path)

    def reset(self, conf=None, data=None, x_columns=None, y_column="", id_fields=list()):
        """
        Set conf, data, x_columns and y_columns. If the value is default, then pass

        Parameters
        --------
        conf : configparser.ConfigParser, default = None
            Configuration information.
        data : pandas.DataFrame, default = None
            Data used for cross-validation.
        x_columns : List, default = None
            Name list of feature fields.
        y_column : str, default = ""
            Name of label field.
        id_fields : list, default = []
            Name list of id_fields.
        """
        if conf is not None:
            self._conf = conf
        if data is not None:
            self._data = data
        if x_columns is not None:
            self._x_columns = x_columns
        if y_column != "":
            self._y_column = y_column
        if len(id_fields) > 0:
            self._id_fields = id_fields
        self._is_executed = False

    def cv_model(self, model):
        """
        Model cross-validation effect.

        Parameters
        --------
        model: model object implementing 'fit'
            The object to use to fit the data.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        if not isinstance(model, base_model.BaseModel):
            self.managerlogger.logger.error("mdoel is not BaseModel")
            return runstatus.RunStatus.FAILED

        try:
            cv_result = []
            all_true_label = []
            all_pre_label = []
            all_prob_label = []
            all_base_info = pd.DataFrame(columns=self._id_fields + [self._y_column])
            try:
                mode = self._conf.get(self.CROSS_VALIDATION, self.VERIFICATION_MODE)
            except TypeError:
                mode = 'k_fold'
            # calculate k parts score
            if mode == 'muti_k_fold':
                for num, data in enumerate(self._cv_data_list):
                    # train and pre
                    try:
                        if self._conf.get(self.CROSS_VALIDATION, self.VALID):
                            model.train((data[0][self._x_columns], data[1][self._x_columns]),
                                        (data[0][self._y_column], data[1][self._y_column]))
                        else:
                            model.train(data[0][self._x_columns], data[0][self._y_column])
                    except TypeError:
                        model.train(data[0][self._x_columns], data[0][self._y_column])
                    # store model
                    cur_test = num // 5
                    pre_prob_result = model.predict_proba(self._muti_cv_list[cur_test][self._x_columns])
                    pre_class_result = model.predict(self._muti_cv_list[cur_test][self._x_columns])

                    # append result
                    all_prob_label.extend(pre_prob_result.tolist())
                    all_pre_label.extend(pre_class_result.tolist())
                    all_true_label.extend(self._muti_cv_list[cur_test][self._y_column].tolist())
                    all_base_info = all_base_info.append(
                        self._muti_cv_list[cur_test][self._id_fields + [self._y_column]])

                    # calculate auc and score
                    model_auc = pre_utils.PredictUtils.get_roc_score(
                        self._muti_cv_list[cur_test][self._y_column].tolist(),
                        pre_prob_result)
                    model_score = pre_utils.PredictUtils.get_model_score(
                        self._muti_cv_list[cur_test][self._y_column].tolist(), pre_class_result)
                    # keep result
                    model_score.update(model_auc)
                    cv_result.append(model_score)
            else:
                for train_data, test_data in self._cv_data_list:
                    # train and pre
                    try:
                        if self._conf.getboolean(self.CROSS_VALIDATION, self.VALID):
                            model.train((train_data[self._x_columns], test_data[self._x_columns]),
                                        (train_data[self._y_column], test_data[self._y_column]))
                        else:
                            model.train(train_data[self._x_columns], train_data[self._y_column])
                    except TypeError:
                        model.train(train_data[self._x_columns], train_data[self._y_column])
                    pre_prob_result = model.predict_proba(test_data[self._x_columns])
                    pre_class_result = model.predict(test_data[self._x_columns])

                    # append result
                    all_prob_label.extend(pre_prob_result.tolist())
                    all_pre_label.extend(pre_class_result.tolist())
                    all_true_label.extend(test_data[self._y_column].tolist())
                    all_base_info = all_base_info.append(test_data[self._id_fields + [self._y_column]])

                    # calculate auc and score
                    model_auc = pre_utils.PredictUtils.get_roc_score(test_data[self._y_column].tolist(),
                                                                     pre_prob_result)
                    model_score = pre_utils.PredictUtils.get_model_score(
                        test_data[self._y_column].tolist(), pre_class_result)

                    # keep result
                    model_score.update(model_auc)
                    cv_result.append(model_score)

            predict_res_df = pd.DataFrame(all_pre_label, columns=[PredictConstance.PRE])
            proba_res_df = pd.DataFrame([str(x) for x in all_prob_label],
                                        columns=[PredictConstance.PROBA])

            # store predict to file
            res = [all_base_info, predict_res_df, proba_res_df]
            [x.reset_index(drop=True, inplace=True) for x in res]
            predict_res = pd.concat(res, axis=1)
            output_file = self._conf.get(self.CROSS_VALIDATION, self.PREDICT_FILE)
            output_file = os.path.join(global_pre.RES_PATH, output_file)
            predict_res.to_csv(output_file, index=False)

            # calculate all parts score
            all_auc = pre_utils.PredictUtils.get_roc_score(all_true_label, all_prob_label)
            all_score = pre_utils.PredictUtils.get_model_score(all_true_label, all_pre_label)

            all_score.update(all_auc)
            cv_result.append(all_score)

            # keep cv result
            output_file = self._conf.get(self.CROSS_VALIDATION, self.RESULT_FILE)
            output_file = os.path.join(global_pre.RES_PATH, output_file)
            with open(output_file, "w") as file_handle:
                file_handle.write("\n".join([str(x) for x in cv_result]))

            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error.error("cv model error:\n %s" % traceback.format_exc())
            self.managerlogger.logger.error("cv model error: %s" % err)
            return runstatus.RunStatus.FAILED

    def execute(self):
        """
        Construct a data set for K-fold cross-validation.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: True
            runstatus.RunStatus.FAILED: Failed
        """
        if self._is_executed:
            return runstatus.RunStatus.SUCC
        if not Cross_Validation._valid_data(self._data):
            return runstatus.RunStatus.FAILED
        try:
            if self._conf.get(self.CROSS_VALIDATION, self.IS_SHUFFLE):
                self._shuf_data()
        except TypeError:
            self.managerlogger.logger.info(
                "%s not found %s, default apply shuffle" % (self.CROSS_VALIDATION, self.IS_SHUFFLE))
        try:
            try:
                mode = self._conf.get(self.CROSS_VALIDATION, self.VERIFICATION_MODE)
                if mode == 'k_fold':
                    self._k_fold_data()
                elif mode == 'group_k_fold':
                    try:
                        group = self._conf.get(self.CROSS_VALIDATION, self.GROUP)
                        if len(group) != 1:
                            self.managerlogger.logger.info("group length must be 1, use default mode kfold")
                            self._k_fold_data()
                        elif group not in self._id_fields:
                            self.managerlogger.logger.info(
                                "%s not found in colums, use default mode kfold" % group)
                            self._k_fold_data()
                        else:
                            self._group_k_fold_data(group)
                    except:
                        self.managerlogger.logger.info(
                            "group not found, use default mode kfold")
                        self._k_fold_data()
                elif mode == 'train_test_split':
                    self._train_test_split()
                elif mode == 'muti_k_fold':
                    self._muti_k_fold_data()
                else:
                    self.managerlogger.logger.info(
                        "%s not found mode name, use default mode kfold" % self.CROSS_VALIDATION)
                    self._k_fold_data()
            except:
                self.managerlogger.logger.info(
                    "mode not found, use default mode kfold")
                self._k_fold_data()
            self._is_executed = True
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("cross validation error: %s" % err)
            self.managerlogger.logger.error.error("cross validation error\n" + traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def _train_test_split(self):
        """
        devide data to only 1 parts according to ratio
        :return:
        """
        try:
            ratio = self._conf.get(self.CROSS_VALIDATION, self.RATIO)
            if ratio <= 0 or ratio >= 1:
                ratio = 0.2
                self.managerlogger.logger.info(
                    "%s must be bigger than 0 and smaller than 1 %s, use default value 0.2" % (
                    self.CROSS_VALIDATION, self.RATIO))
        except TypeError:
            ratio = 0.2
            self.managerlogger.logger.info(
                "%s not found %s, use default value 0.2" % (self.CROSS_VALIDATION, self.RATIO))
        if not Cross_Validation._valid_data(self._data):
            raise ValueError("train data is not valid")
        # devide data
        end_index = int(self._data.shape[0] * ratio)
        train_data = self._data.iloc[:end_index]
        test_data = self._data.iloc[end_index:]
        self._cv_data_list = [(train_data, test_data)]

    def _k_fold_data(self):
        """
        Divide data into k parts.
        """
        try:
            k = int(self._conf.get(self.CROSS_VALIDATION, self.K_FOLD))
            if k <= 1:
                k = 7
                self.managerlogger.logger.info(
                    "%s must be greater than 1 %s, use default value 7" % (self.CROSS_VALIDATION, self.K_FOLD))
        except TypeError:
            k = 7
            self.managerlogger.logger.info(
                "%s not found %s, use default value 7" % (self.CROSS_VALIDATION, self.K_FOLD))
        if not Cross_Validation._valid_data(self._data):
            raise ValueError("train data is not valid")
        # devide data to k parts
        length = self._data.shape[0]
        range_list = [(int((i * length) / k), int(((i + 1) * length) / k)) for i in range(k)]

        # keep k's parts result, [(train, test),]
        self._cv_data_list = []
        for star_index, end_index in range_list:
            test_data = self._data.iloc[star_index:end_index]
            train_data = self._data.iloc[:star_index].append(self._data.iloc[end_index:])
            self._cv_data_list.append((train_data, test_data))

    def _group_k_fold_data(self, group):
        """
        devide data to k parts
        :return:
        """
        try:
            k = int(self._conf.get(self.CROSS_VALIDATION, self.K_FOLD))
            if k <= 1:
                k = 7
                self.managerlogger.logger.info(
                    "%s must be greater than 1 %s, use default value 7" % (self.CROSS_VALIDATION, self.K_FOLD))
        except TypeError:
            k = 7
            self.managerlogger.logger.info(
                "%s not found %s, use default value 7" % (self.CROSS_VALIDATION, self.K_FOLD))
        if not Cross_Validation._valid_data(self._data):
            raise ValueError("train data is not valid")
        # get groups
        group_list = self._data[group].unique()
        group_index = []
        for value in group_list:
            group_index.append(self._data[self._data[group] == value].index)
        # divide data according to group/k
        self._cv_data_list = []
        for i in range(k):
            range_list = []
            for j in range(len(group_list)):
                cur_group_index = group_index[j]
                length = len(cur_group_index)
                range_list.extend(cur_group_index[int((i * length) / k) : int(((i + 1) * length) / k)])
            test_data = self._data.iloc[range_list]
            train_data = self._data.iloc[~self._data.index.isin(range_list)]
            self._cv_data_list.append((train_data, test_data))

    def _muti_k_fold_data(self):
        """
        devide data to k * k parts
        :return:
        """
        try:
            k = int(self._conf.get(self.CROSS_VALIDATION, self.K_FOLD))
            if k <= 1:
                k = 5
                self.managerlogger.logger.info(
                    "%s must be greater than 1 %s, use default value 5" % (self.CROSS_VALIDATION, self.K_FOLD))
        except TypeError:
            k = 5
            self.managerlogger.logger.info(
                "%s not found %s, use default value 5" % (self.CROSS_VALIDATION, self.K_FOLD))
        if not Cross_Validation._valid_data(self._data):
            raise ValueError("train data is not valid")
        # devide data to k parts
        self._cv_data_list = []
        self._muti_cv_list = []
        length = self._data.shape[0]
        range_list = []
        for i in range(k):
            range_list.append((int((i * length) / k), int(((i + 1) * length) / k)))
            new_list = list(range(0, int((i * length) / k)))
            new_list.extend(list(range(int(((i + 1) * length) / k), length)))
            new_length = len(new_list)
            for j in range(k):
                star_index = int((j * new_length) / k)
                end_index = int(((j + 1) * new_length) / k)
                test_data = self._data.iloc[new_list[star_index : end_index]]
                train_data = self._data.iloc[new_list[:star_index]].append(self._data.iloc[new_list[end_index:]])
                self._cv_data_list.append((train_data, test_data))
        for star_index, end_index in range_list:
            test_data = self._data.iloc[star_index:end_index]
            self._muti_cv_list.append(test_data)


    def _shuf_data(self):
        """Shuffle the data set."""
        # self._data = shuffle(self._data)
        try:
            seed = int(self._conf.get(self.CROSS_VALIDATION, self.SEED))
            self._data = self._data.sample(frac=1, random_state=seed).reset_index(drop=True)
        except:
            self._data = self._data.sample(frac=1).reset_index(drop=True)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import pandas as pd
import configparser
import copy

sys.path.append("../../../..")

from xes_ml_arch.src.feature_enginnering import feature_discretizer


class TestDiscretize(unittest.TestCase):
    def setUp(self):
        self._data_path = "../data/test_discretize.txt"
        self._config = "./conf/test_discretize.conf"
        self._data = pd.read_csv(self._data_path)
        self._conf = configparser.ConfigParser()
        self._conf1 = configparser.ConfigParser()
        self._conf.read(self._config)
        self._conf1.read("./conf/test_discretize_try.conf")
        self._ins = feature_discretizer.FeatureDiscretizer(data=self._data, conf=self._conf, log_path = '../log/log.conf')
        self._ins_try = feature_discretizer.FeatureDiscretizer(data=None, conf=self._conf1,log_path = '../log/log.conf')

    def test_excute(self):
        self.assertTrue(self._ins.excute())
        self.assertFalse(self._ins_try.excute())

    def test_get_data(self):
        self._ins.reset(data=self._data, conf=self._conf)
        tmp_data = copy.copy(self._data)
        self._ins.excute()
        data = self._ins.get_data
        # print data
        # print tmp_data
        res = pd.merge(data, tmp_data, on=["userid"], suffixes=["_dis", "_old"])
        print("res:\n", res)
        res_diff = res[res["age_dis"] != res["targ3_dis"]]
        print("res diff:\n", res_diff)


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import json
import datetime
from ..ml_utils import configmanager


class AccessDictModel():
    """Get config profile information"""
    def __init__(self, log_path = None):
        self.acc_Logger_Dict = {}
        self.log_option_list = list()
        self.log_option_property_dict = dict()

        #Configuration file instance
        standard_log_conf = configmanager.ConfigManager('./config/log_schemas.conf', log_path)

        #get options
        for k, v in standard_log_conf.get_keys('default_schema'):
            self.log_option_list.append(k)
            self.log_option_property_dict[k] = v
        self.reset()

    def format_body(self):
        """Convert dictionary type data to JSON type.

        Returns
        -------
        josn string.
        """

        json_str = json.dumps(self.acc_Logger_Dict, default=lambda o: o.__dict__, sort_keys=True, indent=4)
        json_str = json_str.replace('\n', '').replace('\t', '')
        return json_str

    def get_log_dic(self):
        return self.acc_Logger_Dict

    def set_log_dic_key(self, key, value):
        self.acc_Logger_Dict[key] = value
        self.set_log_dic_key_time(key)

    def set_log_dic_key_time(self, key):
        key_time = str(key) + '_time'
        time_value = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        self.acc_Logger_Dict[key_time] = time_value

    def set_data_log_dic_key(self, req_or_resp, key, value):
        for data_dict in self.acc_Logger_Dict['data']:
            if data_dict.get(req_or_resp) is None:
                continue
            else:
                inner_dict = data_dict.get(req_or_resp)
                inner_dict[key] = value
                break

    def reset(self):
        """The configuration information is converted into a dictionary whose value represents the data type of
        the configuration information;For example:'string'== V, self.acc_ Logger_ Dict[k] = ''.

        Returns
        -------
        the dict of configuration information data type.
        """

        for k in self.log_option_property_dict:
            v = self.log_option_property_dict[k]
            if 'int' == v:
                self.acc_Logger_Dict[k] = 0
            elif 'string' == v:
                self.acc_Logger_Dict[k] = ''
            elif 'dict' == v:
                if self.acc_Logger_Dict.get(k) is None:
                    self.acc_Logger_Dict[k] = dict()
                else:
                    self.acc_Logger_Dict[k].clear()
            elif 'list' == v:
                if 'data' == k:
                    if self.acc_Logger_Dict.get(k) is None:
                        self.acc_Logger_Dict[k] = list()
                        request_dict = dict()
                        request_dict["req"] = dict()
                        self.acc_Logger_Dict[k].append(request_dict)
                        response_dict = dict()
                        response_dict["resp"] = dict()
                        self.acc_Logger_Dict[k].append(response_dict)
                    else:
                        for data_dict in self.acc_Logger_Dict[k]:
                            inner_dict = data_dict.get('req')
                            if inner_dict is not None:
                                inner_dict.clear()
                            inner_dict = data_dict.get('resp')
                            if inner_dict is not None:
                                inner_dict.clear()
                else:
                    if self.acc_Logger_Dict.get(k) is None:
                        self.acc_Logger_Dict[k] = list()
                    else:
                        del self.acc_Logger_Dict[k][:]
            else:
                self.acc_Logger_Dict[k] = None



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import unittest
import sys
import random
import configparser
import pandas as pd
import numpy as np
import traceback


sys.path.append('../../../..')
from xes_ml_arch.src.model import model_factory


class TestModelFactory(unittest.TestCase):
    def setUp(self):
        self.model_list = ['lr', 'rf', 'line_reg', 'xgb', 'sklearn_xgb', 'dst', 'lightgbm']
        self.conf_file_lr = "../config/xiaozhuan_prediction_lr.conf"
        self.conf_file_rf = "../config/xiaozhuan_prediction_rf.conf"
        self.conf_file_xgb = "../config/xiaozhuan_prediction_xgb.conf"
        self.conf_file_sklearn_xgb = "../config/xiaozhuan_prediction_sklearn_xgb.conf"
        self.conf_file_line_reg = "../config/xiaozhuan_prediction_linereg.conf"
        self.conf_file_lightgbm = "../config/xiaozhuan_prediction_lightgbm.conf"
        self.x = pd.read_csv('../data/test_aaa_test.txt', sep=',')
        self.n_x = None
        self.n_y = None
        self.e_x = pd.read_csv('../data/test_aaa_test.txt', sep=',')
        # self.y = pd.DataFrame([int(random.random() * 100) for x in range(999)])
        self._y = np.random.randint(0,3,size=(99,))
        self._y.reshape(1, 99)
        self.y = np.array([random.random() for x in range(99)])
        #print(self._y,self.y)
        self.y.reshape(1, 99)

    def test_model(self):
        conf_file = configparser.ConfigParser()
        conf_file.read('../config/model.conf')
        for name in self.model_list:
            # file = 'self.conf_file_' + name
            # conf_file.read(eval(file))

            # print type(conf_file)
            # print conf_file.sections()
            model = model_factory.ModelFactory.create_model(conf_file, name, log_path = '../log/log.conf')
            if name == 'xgb':
                self.assertTrue(model.train(self.x, self.y))
                print("xgb init successed")
            else:
                print("%s 开始测试" % (name))
                self.assertTrue(model.train(self.x, self._y))

            self.assertIsNotNone(model.predict(self.x))
            if name == 'line_reg':
                pass
            else:
                self.assertIsNotNone(model.predict_proba(self.x))
            self.assertIsNotNone(model.get_feature_importance(self.x))
            self.assertTrue(model.store_model('../config/zscal_pickle_file_lr.properties'))
            self.assertTrue(model.load_model('../config/zscal_pickle_file_lr.properties'))
            if name == "lr":
                self.assertTrue(model.store_feature_importance(self.x))
                print("lr init successed")
            if name == "dst":
                model.show_dot('./dst', self.x)
                print("决策树流程结构图创建")

    def test_model_exception(self):
        conf_file = configparser.ConfigParser()
        conf_file.read("../config/model_try.conf")
        nofile = None
        feature = None
        model1 = model_factory.ModelFactory.create_model(conf_file, log_path = '../log/log.conf')
        for name in self.model_list:
            print('name: --------',name)
            model = None
            # file = 'self.conf_file_' + name
            # conf_file.read(eval(file))
            try:
                if name not in ['line_reg','xgb']:
                    model = model_factory.ModelFactory.create_model(conf_file, name, log_path= '../log/log.conf')
                    self.assertFalse(model.train(self.e_x, self.y))
                    self.assertFalse(model.load_model('../config/zscal_pickle_file'+'_%s.pickle'%(name)))
                    self.assertFalse(model.store_feature_importance(self.n_x))
                    self.assertIsNone(model.predict(self.x))
                    self.assertIsNone(model.get_feature_importance(feature))
                    self.assertTrue(model.store_model(nofile))
                else:
                    #line_reg No matter what the data format is, it can be trained successfully
                    #xgb obj:binary  train false while y is Continuous categories（int）
                    model = model_factory.ModelFactory.create_model(conf_file, name, log_path= '../log/log.conf')
                    self.assertFalse(model.train(self.e_x, self._y))
                    self.assertFalse(model.load_model('../config/zscal_pickle_file' + '_%s.pickle' % (name)))
                    self.assertFalse(model.store_feature_importance(self.n_x))
                    self.assertIsNone(model.predict(self.x))
                    self.assertIsNone(model.get_feature_importance(feature))
                    self.assertTrue(model.store_model(nofile))
                if name == 'line_reg':
                    pass
                else:
                    self.assertIsNone(model.predict_proba(self.x))
            except:
                traceback.print_exc()
            # model.get_feature_importance(self.n_x)

            try:
                #test create_model function test: the parameter passed in when creating a model can only be config
                model = model_factory.ModelFactory.create_model('../config/zscal_pickle_file', name)
            except:
                traceback.print_exc()

            # model = model_factory.ModelFactory.create_model(nofile, name)
        try:
            model = model_factory.ModelFactory.create_model(conf_file, 'sklearn_xgb', log_path= '../log/log.conf')
        except:
            traceback.print_exc()


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import configparser

from ..systemlog import sysmanagerlog
from ..systemlog import syserrorlog


class ConfigManager():
    """Reading of log system synchronization configuration file"""

    def __init__(self, configfile, log_path = None):
        """
        Initialization parameter.

        Parameters
        --------
        configfile: System Configuration file path of user.
        """

        self.configfile = configfile
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__,log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__,log_path)
        self.configP = self.init_config(self.configfile, self.managerlogger)

    def init_config(self, configfile, logger):
        """
        Loading configuration information.

        Parameters
        ----------
        configfile：System Configuration file path

        Returns
        -------
        Configuration information file.
        """
        self.managerlogger.logger.info('Start innit the config.....')

        conf = configparser.ConfigParser()
        conf.read(configfile)

        self.managerlogger.logger.info('End init the config.....')
        return conf

    def get_key(self, group, key):
        """Gets the option value of the named part"""
        return self.configP.get(group, key)

    def get_keys(self, group):
        """Gets the content of the configuration file section(group)；
            contens: tuple of list.

        Parameters
        ----------
        group: section name of configuration file.

        Returns
        -------
        tuple of list
        """

        return self.configP.items(group)

    def get_sections(self):
        return self.configP.sections()

    def get_float(self, section, option):
        return self.configP.getfloat(section, option)

    def has_option(self, section, option):
        return self.configP.has_option(section, option)

    def __iter__(self):
        for section in self.configP.sections():
            yield section



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import json

def get_message(message):
    """Generating Python objects into JSON objects. Element of json is string.

    Parameters
    ----------
    message: input date,dict.

    Returns
    -------
    date of json.
    """
    # json_str = json.dumps(message, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    # json_str = json_str.replace('\n', '').replace('\t', '')
    json_str = json.dumps(message, default=lambda o: o.__dict__, sort_keys=True)
    return json_str


def get_message_without_whitespace(message):
    """Generating Python objects into JSON objects with no space. Element of json is string.

    Parameters
    ----------
    message: input date,dict.

    Returns
    -------
    date of json.
    """

    json_str = json.dumps(message, default=lambda o: o.__dict__, sort_keys=True)
    json_str = json_str.replace('\n', '').replace('\t', '').replace(' ', '')
    return json_str



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import random
import pandas as pd
import numpy as np

sys.path.append('../../../..')
from xes_ml_arch.src.ml_utils import pre_utils
from xes_ml_arch.src.ml_utils import feature_processor
from xes_ml_arch.src.ml_utils import jsonmanager


class TestUtils(unittest.TestCase):
    def setUp(self):
        self.pre_utils = pre_utils.PredictUtils()
        self.int_label = np.random.randint(0, 9, 99)
        self.label = np.random.randint(0, 9, 99)
        self.long_label = np.random.randint(0, 9, 100)
        self.x = [[random.random() * 100 for x in range(100)] for y in range(30)]
        self.bi_label = np.random.randint(0, 2, 99)
        self.multi_label = np.random.randint(0, 5, 99)
        self.bi_pre_label = np.random.randint(0, 2, 99)
        self.prob_label = np.random.uniform(0, 1, size=(99, 2))
        self.multi_prob_label = np.random.uniform(0, 1, size=(99, 5))
        self.prob_long_label = np.random.uniform(0, 1, size=(100, 2))
        self.prob_label.tolist()
        # self.no_list_x = pd.DataFrame(np.random.randn(2, 4))
        self.no_label = None

    def test_feature_processor(self):
        at = 1
        bt = 1.0
        ct = [1.0]
        resa = feature_processor.FeatureProcessor().convert_to_int(at)
        resb = feature_processor.FeatureProcessor().convert_to_int(bt)
        self.assertTrue(isinstance(resa, int))
        self.assertTrue(isinstance(resb, int))
        resc = feature_processor.FeatureProcessor().convert_to_int(ct)
        self.assertEqual(resc, 0)

    def test_jsonmanager(self):
        test_dic = {"a": 1,  "b": 2,  "d": 4,  "c": 3,  "e": 5,  'f': 'ewd see'}
        res1 = jsonmanager.get_message(test_dic)
        print(res1)
        self.assertTrue(isinstance(res1, str))
        res2 = jsonmanager.get_message_without_whitespace(test_dic)
        print(res2)
        self.assertTrue(isinstance(res1, str))

    def test_store_feature_importance(self):
        no_x = None
        res = self.pre_utils.store_feature_importance(self.x, '../data/test_store_feature_importance.txt')
        self.assertIsNotNone(res)
        try:
            res_no = self.pre_utils.store_feature_importance(no_x, '../data/test_store_feature_importance.txt')
        except:
            res_no = None

    def test_get_precision(self):
        res1 = self.pre_utils.get_precision(self.label, self.int_label)
        self.assertIsNotNone(res1)
        res2 = self.pre_utils.get_precision(list(self.label), list(self.int_label))
        self.assertIsNotNone(res2)

        try:
            self.pre_utils.get_precision(self.no_label, self.label)
        except:
            print("precison:label type can't be converted to a list")
        try:
            self.pre_utils.get_precision(self.label, self.no_label)
        except:
            print("precison:label type can't be converted to a list")
        try:
            self.pre_utils.get_precision(self.label, self.long_label)
        except:
            print("precison: length of label and pre_label is not equal")

    def test_get_model_score(self):
        res1 = self.pre_utils.get_model_score(self.label, self.int_label)
        self.assertTrue(isinstance(res1, dict))
        res2 = self.pre_utils.get_model_score(self.bi_label, self.bi_pre_label)
        self.assertTrue(isinstance(res2, dict))
        res3 = self.pre_utils.get_model_score(list(self.bi_label), list(self.bi_pre_label))
        self.assertTrue(isinstance(res3, dict))
        try:
            self.pre_utils.get_model_score(self.label, self.long_label)
        except:
            print("model_score: length of label and pre_label is not equal")
        try:
            self.pre_utils.get_model_score(self.no_label, self.label)
        except:
            print("model_score:label type can't be converted to a list")
        try:
            self.pre_utils.get_model_score(self.label, self.no_label)
        except:
            print("model_score:label type can't be converted to a list")

    def test_get_roc_score(self):
        res1 = self.pre_utils.get_roc_score(self.bi_label, self.prob_label)
        self.assertTrue(isinstance(res1, dict))
        res2 = self.pre_utils.get_roc_score(list(self.bi_label), list(self.prob_label))
        self.assertTrue(isinstance(res2, dict))
        res3 = self.pre_utils.get_roc_score(self.bi_label, np.array(self.prob_label))
        self.assertTrue(isinstance(res3, dict))
        res4 = self.pre_utils.get_roc_score(self.multi_label, np.array(self.multi_prob_label))
        self.assertTrue(isinstance(res4, dict))
        self.pre_utils.get_roc_score(self.multi_label, np.random.uniform(0, 1, size=(99, 5)))

        try:
            self.pre_utils.get_roc_score(self.bi_label, np.array(self.prob_long_label))
        except:
            print("roc_socore:length of label and pre_label is not equal")

        try:
            self.pre_utils.get_roc_score(self.multi_label, np.array(self.prob_label))
        except:
            print("roc_socore:length of label and pre_label is not equal")
        try:
            self.pre_utils.get_roc_score(self.no_label, self.label)
        except:
            print("roc_socore:label type can't be converted to a list")
        try:
            self.pre_utils.get_roc_score(self.label, self.no_label)
        except:
            print("roc_socore:label type can't be converted to a list")


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import pandas as pd
import numpy as np
import traceback
from ..systemlog import sysmanagerlog
from ..systemlog import syserrorlog


class FeatureDiscretizer(object):
    """
    Discretize feature.

    Parameters
    ----------
    data : pandas.DataFrame, default = None
        origin dataset
    conf: configparser.ConfigParser, default = None
        Configuration file

    Examples
    --------
    >>> from xes_ml_arch.src.feature_enginnering import feature_discretizer
    >>> import configparser
    >>> import pandas as pd
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> data = pd.read_csv("my_data.csv")
    # this
    >>> ins = feature_discretizer.FeatureDiscretizer(data=data, conf=conf)
    >>> ins.excute()
    """

    DIS_FUNC = "dis_func"
    ARGS = "args"

    EQ_FREQUE = "eq_freque"
    EQ_FREQUE_VALUE = "eq_freque_value"
    EQ_VALUE = "eq_value"

    FEATURE_DISCRETIZE = "feature_discretize"
    PARAMS = "params"

    def __init__(self, data=None, conf=None, log_path = None):
        self._data = data
        self._conf = conf
        self._feature_conf = {}
        self.managerlogger = sysmanagerlog.SysManagerLog(__file__, log_path)
        self.errorlogger = syserrorlog.SysErrorLog(__file__, log_path)

    def _init(self):
        """
        Initialization.

        Returns
        --------
        :return: bool
            True: succ
            False: failed
        """
        try:
            if self.FEATURE_DISCRETIZE not in self._conf.sections():
                return False
            # load config
            self._feature_conf = eval(self._conf.get(self.FEATURE_DISCRETIZE, self.PARAMS))
            self.managerlogger.logger.info("feature discretizer init succeed")
            return True
        except Exception as e:
            self.managerlogger.logger.error("Feature Discretizer error: %s" % e)
            self.errorlogger.logger.error("Feature Discretizer error:\n %s" % traceback.format_exc())
            return False

    def _discretize(self):
        """
        Discretize features based on configuration

        Returns
        --------
        :return: bool
            True: succ
            False: failed
        """
        try:
            data_columns = self._data.columns.tolist()
            for key in self._feature_conf:
                if key not in data_columns:
                    continue
                # Use different discretization functions according to configuration.
                func = self._discretize_freque
                if self._feature_conf[key][self.DIS_FUNC] == self.EQ_FREQUE:
                    func = self._discretize_freque
                elif self._feature_conf[key][self.DIS_FUNC] == self.EQ_VALUE:
                    func = self._discretize_value
                elif self._feature_conf[key][self.DIS_FUNC] == self.EQ_FREQUE_VALUE:
                    func = self._discretize_freque_value

                self._data[key] = pd.Series(func(self._data[key].tolist(), self._feature_conf[key][self.ARGS]))
            self.managerlogger.logger.info("feature discretize succeed")
            return True
        except Exception as e:
            self.managerlogger.logger.error("feature discretize error: %s" % e)
            self.errorlogger.logger.error("feature discretize error:\n %s" % traceback.format_exc())
            return False

    def _discretize_freque(self, value_list, args):
        """
        Equal frequency division, evenly divided into multiple segments.

        Parameters
        --------
        value_list:
            Columns that need to be discretized in the data set
        args: int
            Number of discrete classes.

        Returns
        --------
        tmp_res : list
            Discretization result

        Examples
        --------
        >>>value_list = np.random.randint(2, 10, 6)
            array([7, 3, 4, 2, 9, 8])
        >>>args = 2
        >>>self._discretize_freque(value_list, args)
            [1, 0, 0, 0, 1, 1]
        """
        length = len(value_list)
        tmp_res = [[index, value_list[index], 0] for index in range(length)]
        tmp_res.sort(key=lambda x: x[1])
        class_nums = int(float(args))
        res = [int((x * class_nums) / length) for x in range(length)]
        for index in range(len(tmp_res)):
            tmp_res[index][2] = res[index]

        tmp_res.sort(key=lambda x: x[0])
        tmp_res = [x[2] for x in tmp_res]

        return tmp_res

    def _discretize_freque_value(self, value_list, args):
        """
        Equal frequency division, different from the above equal frequency division, the same
        value can be divided into the same category.

        Parameters
        --------
        value_list:
            Columns that need to be discretized in the data set
        args: int
            Number of discrete classes.

        Returns
        --------
        :return: list
        """
        length = len(value_list)
        tmp_res = [[index, value_list[index], 0] for index in range(length)]
        tmp_res.sort(key=lambda x: x[1])
        class_nums = int(float(args))

        # Divide equally into n segments and take the value of the node in each segment.
        step = int(length / class_nums)
        bins = [tmp_res[x][1] for x in range(step, length, step)]

        return np.digitize(value_list, bins=bins).tolist()

    def _discretize_value(self, value_list, args):
        """
        Discretize according to the given threshold.

        Parameters
        --------
        value_list: list
        args: list

        Returns
        --------
        :return: list

        Examples
        --------
        >>> a = [7, 3, 4, 2, 9, 8]
        >>> args = [4, 7, 10]
        >>>self._discretize_value(a, args)
            [2, 0, 1, 0, 2, 2]
        """
        return np.digitize(value_list, bins=args).tolist()

    def reset(self, data=None, conf=None):
        """
        Reset data and conf.

        Parameters
        --------
        data: pandas,DataFrame
        conf: ConfigParser.ConfigParser
        """
        if not isinstance(data, pd.DataFrame):
            self.managerlogger.logger.error("data is not dataframe")
            return False
        self._data = data
        self._conf = conf

    def excute(self):
        """
        Excute Member method,self._init and self._discretize.

        Returns
        --------
        :return: bool
            True: succ
            False: failed
        """
        if not self._init():
            return False
        return self._discretize()

    @property
    def get_data(self):
        """
        useage: feature_discretizer = FeatureDiscretizer()
                data = feature_discretizer.get_data

        Returns
        --------
        self._data
        """
        return self._data



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from . import base_ml
import traceback
from ..ml_utils import runstatus


class PredictionML(base_ml.BaseML):
    """
    This basic class encapsulates the functions of the prediction part, and you can call the method
    of the class to make predictions on the test set.

    Parameters
    --------
    conf : configparser.ConfigParser, default = None
        Configuration file for prediction of the test data set.

    Examples
    --------
    >>> from xes_ml_arch.src.ml import prediction_ml
    >>> import configparser
    >>> import pandas as pd
    >>> conf = configparser.ConfigParser()
    >>> conf.read("myconfig.conf")
    >>> pml = prediction_ml.PredictionML(conf=conf)
    >>> data = pd.read_csv("my_data.csv")
    >>> pml.set_data(data)
    >>> pml.start()
    """

    def __init__(self, conf=None,xeasy_log_path = None):
        self._test_data = None
        super(PredictionML, self).__init__(config=conf, xeasy_log_path = xeasy_log_path)

    def start(self):
        """
        Start predict data handle.
        """
        self.managerlogger.logger.info("start ml predict...")
        if runstatus.RunStatus.SUCC == self._predict_handle():
            self.managerlogger.logger.info("finished ml predict!")
        else:
            self.managerlogger.logger.error("ml predict failed!")

    def _init_model(self):
        """
        Load the trained model.

        Returns
        -------
        :return: bool
            True : Succ
            False : failed
        """
        if not super(PredictionML, self)._init_model():
            return False
        # load model
        if runstatus.RunStatus.FAILED == self._model.load_model():
            self.managerlogger.logger.error("load model error")
            return False
        self.managerlogger.logger.info("successfly load model to predict: %s" % self._model.MODEL_ID)
        return True

    def _predict_handle(self):
        '''
        Model predict handle.

        Returns
        -------
        :return: bool
            True : Succ
            False : failed
        '''
        try:
            self._feature_processor.test_data = self._data
            if runstatus.RunStatus.FAILED == self._feature_processor.execute():
                self.managerlogger.logger.error("predict feature processor error")
                return False
            self.managerlogger.logger.info("successfly predict model: %s" % self._model.MODEL_ID)
            # get predict result
            if runstatus.RunStatus.FAILED == self._get_result():
                self.managerlogger.logger.error("predict get result error")
                return False
            self.managerlogger.logger.info("successfly get result of predict : %s" % self._model.MODEL_ID)

            # store result to file
            if runstatus.RunStatus.FAILED == self._store_predict_result():
                self.managerlogger.logger.error("store predict result error")
                return False
            self.managerlogger.logger.info("successfly store result of predict : %s" % self._model.MODEL_ID)
            return True

        except:
            self.managerlogger.logger.debug(traceback.format_exc())
            self.managerlogger.logger.error("predict handle error")
            return False


# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import os
import sys
from ..tttttest import w
print(os.getcwd())
import configparser
conf = configparser.ConfigParser()
conf.read('./ml.conf')
print(os.path.dirname(__file__) + os.sep + '../')
#当文件不是包或者定义在项目内时；如果我们要用包的相对路径导入其他包，需要在系统路径内加入当前py文件需导入的文件的相对路径（部分）
print(sys.path)

#sys.path.append(os.path.dirname(__file__) + os.sep + '../')
sys.path.append('../') #将当前文件所在的目录的上级目录加入sys，这是个全局的，当前文件的意思就是这个py文件，不是最初的启动文件
print(sys.path)
#相比于刚才增加了一个路径


#测试,路径里包含了test_path 的上级目录，所以直接导入下面的文件的话，系统会自己去找
import test_ml_predict




# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import pandas as pd
import time
import datetime
import numpy as np
from ..ml_utils.global_pre import Global


class FeatureProcessor(object):
    """Handle one feature."""

    LOCATION = "feature_processor.FeatureProcessor."
    TIME_FORM = "%Y-%m-%d %H:%M:%S"

    @staticmethod
    def convert_to_int(param):
        """Convert one feature to int.

        Parameters
        ----------
        param: input data to be converted.

        Returns
        -------
        if param's type is int, return param; else if int(param) can be implement return int(param);
        else return 0.

        Examples
        -------
        >>>param = 4
        >>>convert_to_int(param)
        4

        >>>param = 4.4
        >>>convert_to_int(param)
        4

        >>>param = 'test'
        >>>convert_to_int(param)
        0
        """
        if isinstance(param, int):
            return param
        else:
            try:
                return int(param)
            except:
                return 0

    @staticmethod
    def convert_to_str(param):
        """Convert one feature to str.

        Parameters
        ----------
        param: input data to be converted.

        Returns
        -------
        if param's type is str, return param; else if str(param) can be implement return str(param);
        else return ''.

        Examples
        -------
        >>>param = 'test'
        >>>convert_to_str(param)
        'test'

        >>>param = 3
        >>>convert_to_str(param)
        '3'

        >>>param = True
        >>>convert_to_str(param)
        ''
        """

        if isinstance(param, str):
            return param
        else:
            try:
                return str(param)
            except:
                return ''

    @staticmethod
    def time2stamp(data):
        """Time convert to timestamp.

        Parameters
        -------
        data: 1d array-like; Every element in it is a time series string.
                pandas.core.series.Series.

        Returns
        -------
        list of Seconds to represent the floating-point number of time.

        Examples
        -------
        >>> data[i] = "2019-02-18 23:40:00"
        >>> time2stamp(data)[i]
        1550504400.0
        """

        if not isinstance(data, pd.core.series.Series):
            return None
        for i in range(len(data)):
            try:
                # 转换成时间数组
                timeArray = time.strptime(str(data[i]), "%Y-%m-%d %H:%M:%S")
                # 转换成时间戳
                timestamp = time.mktime(timeArray)
                data[i] = timestamp
            except:
                data[i] = None
        return data

    @staticmethod
    def stamp2time(data):
        """Timestamp convert to time.

        Parameters
        -------
        data:  A list of timestamp date(float).

        Returns
        -------
        A list of time(str).

        Examples
        -------
        >>> data[i] = 1550504400.0
        >>> stamp2time(data)[i]
        "2019-02-18 23:40:00"
        """

        if not isinstance(data, pd.core.series.Series):
            return None
        try:
            data = data.astype('int')
        except:
            return
        for i in range(len(data)):
            try:
                # 转换成localtime
                time_local = time.localtime(int(data[i]))
                # 转换成新的时间格式(2016-05-05 20:28:54)
                dt = time.strftime(Global.TIME_FORM, time_local)
                data[i] = dt
            except:
                data[i] = None
        return data

    @staticmethod
    def time_diff(data1, data2):
        """Real time convert to timastamp then Calculating the difference.

        Parameters
        -------
        data1: 1d arrat-like; Every element in it is a time series string.
                pandas.core.series.Series
        data2: 1d arrat-like; Every element in it is a time series string.
                pandas.core.series.Series

        Returns
        -------
        _diff: 1d array-like; Time difference of corresponding time(float).
                np.array.

        Examples
        -------
        >>>data1 = pd.date_range('20200101',periods=6, freq='1h30min')
        >>>data2 = pd.date_range('20190101',periods=6, freq='1h45min')
        >>>time_diff(data1, data2)
        array([31536000., 31535100., 31534200., 31533300., 31532400., 31531500.])
        """

        if not (isinstance(data1, pd.core.series.Series) and
                isinstance(data2, pd.core.series.Series)):
            return None
        if len(data1) != len(data2):
            return None
        _diff = np.zeros(len(data1))
        for i in range(len(data1)):
            try:
                # 转换成时间数组
                timeArray1 = time.strptime(str(data1[i]), Global.TIME_FORM)
                # 转换成时间戳
                timestamp1 = time.mktime(timeArray1)
                # 转换成时间数组
                timeArray2 = time.strptime(str(data2[i]), Global.TIME_FORM)
                # 转换成时间戳
                timestamp2 = time.mktime(timeArray2)
                _diff[i] = int(abs(int(timestamp1) - int(timestamp2)))
            except:
                _diff[i] = None
        return _diff

    @staticmethod
    def minus_data(data1, data2):
        """Time difference.

        Parameters
        ----------
        data1: 1d arrat-like; Every element in it is a timestamp float.
                pandas.core.series.Series
        data2: 1d arrat-like; Every element in it is a timestamp float.
                pandas.core.series.Series

        Returns
        -------
        result data1 - data2
        """
        if not (isinstance(data1, pd.core.series.Series) and
                isinstance(data2, pd.core.series.Series)):
            return None
        if len(data1) != len(data2):
            return None

        _res = np.zeros(len(data1))
        for i in range(len(data1)):
            try:
                _res[i] = float(data1[i]) - float(data2[i])
            except:
                _res[i] = None
        return _res

    @staticmethod
    def abs_minus_data(data1, data2):
        """Absolute value of time difference.

        Parameters
        ----------
        data1: 1d arrat-like; Every element in it is float.
                pandas.core.series.Series
        data2: 1d arrat-like; Every element in it is float.
                pandas.core.series.Series

        Returns
        -------
        result |data1 - data2|
        """
        if not (isinstance(data1, pd.core.series.Series) and isinstance(data2,
                                                                        pd.core.series.Series)):
            return None
        if len(data1) != len(data2):
            return None
        _res = np.zeros(len(data1))
        for i in range(len(data1)):
            try:
                _res[i] = abs(float(data1[i]) - float(data2[i]))
            except:
                _res[i] = None
        return _res

    @staticmethod
    def stay_in_year(years):
        """Converts str(years type) to int.

        Parameters
        ----------
        years:1d array-like; Every element in it is str with '+'.
                pandas.core.element.Series.

        Returns
        -------
        res: 1d array-like; Every element in it is a year series int while year can be convert to int else 0.
              pandas.core.series.Series.
        """

        res = []
        for year in years:
            try:
                res.append(int(year.replace("+", "")))
            except:
                return res.append(0)
        return res

    @staticmethod
    def age2int(ages):
        """Convert str(age type) to int.

        Parameters
        ----------
        ages: 1d array-like;Every element in it is string.
             pandas.core.series.Series.

        Returns
        -------
        res: 1d array-like;Every element in it is int.
             pandas.core.series.Series.
        """

        res = []
        for age in ages:
            try:
                if "-" in age:
                    age = [int(x) for x in age.split("-")]
                    res.append(float(sum(age)) / len(age))
                elif "+" in age:
                    res.append(int(age.replace("+", "")))
                else:
                    res.append(int(age))
            except:
                res.append(-1)
        return res

    @staticmethod
    def discretize_freque(value_list, args=3):
        """Segment evenly according to the input value.

        Parameters
        ----------
        value_list: 1d array-like; pandas.core.series.Series.
        args: Segment parameter(int).

        Returns
        -------
        Segment label; 1d array-like.

        Examples
        -------
        >>>list = [2,345345,34,5,34,53,4,3,53,45,3,5]
        >>>discretize_freque(list,args = 3)
        [0, 2, 1, 1, 1, 2, 0, 0, 2, 2, 0, 1]
        """
        value_list = list(value_list)
        length = len(value_list)
        tmp_res = [[index, value_list[index], 0] for index in range(length)]
        tmp_res.sort(key=lambda x: x[1])
        class_nums = int(float(args))
        res = [int((x * class_nums) / length) for x in range(length)]
        for index in range(len(tmp_res)):
            tmp_res[index][2] = res[index]

        tmp_res.sort(key=lambda x: x[0])
        tmp_res = [x[2] for x in tmp_res]

        return tmp_res

    @staticmethod
    def get_day_range(day_num):
        """ Range of days.

        Parameters
        ----------
        day_num: max days.

        Returns
        -------
        max range of days. int.
        """
        try:
            day = int(day_num)
            if 7 < day < 31:
                return 30
            elif day <= 7:
                return day
            elif 31 <= day < 366:
                return 365
            elif 366 <= day < 731:
                return 730
            else:
                return 1095
        except:
            return -1



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from .base_model import BaseModel
from .base_model import SklearnModel
from .lr import LR
from .my_xgb import MyXgb
from .rf import RF
from .sklearn_xgb import SklearnXGB
from .sklearn_xgb_reg import SklearnXGBReg

from . import model_factory


# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import unittest
import sys
import random
import configparser
import pandas as pd
import numpy as np
import traceback

sys.path.append('../../../..')
from xeasy_ml.xes_ml_arch.src.model import lr
from xeasy_ml.xes_ml_arch.src.model import rf
from xeasy_ml.xes_ml_arch.src.model import linear
from xeasy_ml.xes_ml_arch.src.model import my_xgb
from xeasy_ml.xes_ml_arch.src.model import sklearn_xgb
from xeasy_ml.xes_ml_arch.src.model import desion_tree
from xeasy_ml.xes_ml_arch.src.model import my_lightgbm
from xeasy_ml.xes_ml_arch.src.model import sklearn_xgb_reg



class TestModelFactory(unittest.TestCase):
    def setUp(self):
        self.conf_file = configparser.ConfigParser()
        self.conf_file.read('../config/model.conf')
        self.conf_file_lr = "../config/xiaozhuan_prediction_lr.conf"
        self.conf_file_rf = "../config/xiaozhuan_prediction_rf.conf"
        self.conf_file_xgb = "../config/xiaozhuan_prediction_xgb.conf"
        self.conf_file_sklearn_xgb = "../config/xiaozhuan_prediction_xgb.conf"
        self.conf_file_line_reg = "../config/xiaozhuan_prediction_linereg.conf"
        self.x = pd.read_csv('../data/test_aaa_test.txt', sep=',')
        self.path = '../log/log.conf'
        self.columns = self.x.columns
        # self.y = pd.DataFrame([int(random.random() * 100) for x in range(999)])
        self._y = np.array([int(random.random() * 100) for x in range(99)])
        self._y.reshape(1, 99)
        self.y = np.random.randint(0,3,size=(99,))
        self.y.reshape(1, 99)

    def test_model_lr(self):
        test_lr = lr.LR(self.conf_file, log_path = self.path)
        if test_lr._init_model():
            self.assertTrue(test_lr.train(self.x, self._y))

    def test_model_rf(self):
        test_rf = rf.RF(self.conf_file, log_path = self.path)
        test_rf._init_model()
        self.assertTrue(test_rf.train(self.x, self._y))
        self.assertTrue(test_rf.get_feature_importance(self.columns))

    def test_model_line_reg(self):
        test_linear = linear.Liner(self.conf_file, log_path = self.path)
        test_linear._init_model()
        self.assertTrue(test_linear.train(self.x, self._y))
        res = test_linear.get_feature_importance(self.x)
        self.assertIsNotNone(res)

    def test_model_xgb(self):
        test_lightgbm = my_xgb.MyXgb(self.conf_file, log_path = self.path)
        test_lightgbm._init_model()
        self.assertTrue(test_lightgbm.train(self.x, self._y))
        res = test_lightgbm.predict(self.x)
        self.assertIsNotNone(res)
        # print res
        self.assertIsNotNone(test_lightgbm.predict_proba(self.x))
        # print res1
        self.assertIsNotNone(test_lightgbm.get_feature_importance(self.x))

    def test_model_sklearn_xgb(self):
        test_model_sklearn_xgb = sklearn_xgb.SklearnXGB(self.conf_file, log_path = self.path)
        test_model_sklearn_xgb._init_model()
        self.assertTrue(test_model_sklearn_xgb.train(self.x, self.y))
        res = test_model_sklearn_xgb.predict(self.x)
        self.assertIsNotNone(res)
        # print res
        res1 = test_model_sklearn_xgb.predict_proba(self.x)
        self.assertIsNotNone(res1)
        res2 = test_model_sklearn_xgb.get_feature_importance(self.x)
        self.assertIsNotNone(res2)

    def test_model_dst(self):
        test_model_dst = desion_tree.MyDesionTree(self.conf_file, log_path = self.path)
        test_model_dst._init_model()
        self.assertTrue(test_model_dst.train(self.x, self._y))
        res = test_model_dst.get_feature_importance(self.x)
        self.assertIsNotNone(res)
        try:
            test_model_dst.show_dot('./desion_tree', res)
        except:
            traceback.print_exc()

    def test_model_lightgbm(self):
        test_lightgbm = my_lightgbm.MyLightGBM(self.conf_file, log_path = self.path)
        test_lightgbm._init_model()
        self.assertTrue(test_lightgbm.train(self.x, self._y))
        res = test_lightgbm.predict(self.x)
        self.assertIsNotNone(res)
        # print res
        self.assertIsNotNone(test_lightgbm.predict_proba(self.x))
        # print res1
        self.assertIsNotNone(test_lightgbm.get_feature_importance(self.x))

    def test_model_sklearn_xgb_reg(self):
        test_model_sklearn_xgb = sklearn_xgb_reg.SklearnXGBReg(self.conf_file, log_path = self.path)
        test_model_sklearn_xgb._init_model()
        self.assertTrue(test_model_sklearn_xgb.train(self.x, self.y))
        res = test_model_sklearn_xgb.predict(self.x)
        self.assertIsNotNone(res)
        # print res
        res1 = test_model_sklearn_xgb.predict_proba(self.x)
        self.assertIsNotNone(res1)
        res2 = test_model_sklearn_xgb.get_feature_importance(self.x)
        self.assertIsNotNone(res2)



if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from .base_model import SklearnModel
from sklearn.linear_model import LinearRegression
import time
import traceback
from ..ml_utils import runstatus


class Liner(SklearnModel):
    """A liner classifier. The encapsulation form is sklearn model.

    Parameters
    --------
    config: configparser.ConfigParser()
        Configuration file for model initialization.

    Examples
    --------
    >>> from xes_ml_arch.src.model import linear
    >>> import configparser
    >>> import pandas as pd
    >>> import numpy as np
    >>> import random
    >>> config = configparser.ConfigParser()
    >>> config.read("liner.conf")
    >>> x = pd.read_csv('data.txt', sep=',')
    >>> y = np.array([int(random.random() * 100) for _x in range(99)])
    >>> ins = linear.Liner(config=config)
    >>> ins.init()
    >>> ins.train(x, y)
    ...
    ...
    """

    def __init__(self, config=None, log_path = None):
        super(Liner, self).__init__(config=config, log_path = log_path)
        self._n_estimators = 200
        self._max_feature = "auto"
        self._max_deepth = None

    def _init_model(self):
        """Initialize model.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """

        try:
            self._init_config()
            self._model = LinearRegression(**self._model_params)
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("init linear model error: %s" % err)
            self.errorlogger.logger.error("init linear model error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def _init_config(self):
        pass

    def train(self, x, y):
        """Train a model use a linear model.

        Parameters
        --------
        x: pandas.DataFrame of shape (n_sample, n_features)
            sample data
        y: array-like of shape (n_samples,)
            Target vector relative to X.

        Returns
        --------
        :return: bool
            True: suceess
            False: faild
        """
        try:
            t_start = time.time()
            self.managerlogger.logger.info("start linear..")
            self._model.fit(x, y)
            self.managerlogger.logger.info("finished liner!")
            t_end = time.time()
            self.managerlogger.logger.info("liner train time: %s " % (t_end - t_start))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("linear train error: %s " % err)
            self.errorlogger.logger.error("linear train error:\n %s " % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def get_feature_importance(self, feature):
        """Get weights of user features.

        Parameters
        --------
        feature: list
            List of feature names.

        Returns
        --------
        res: dict
            Feature importance, like : dict([feature_importance, feature name])

        """
        try:
            res = zip(self._model.coef_, feature)
            return res
        except Exception as err:
            self.managerlogger.logger.error("linear get feature importance error: %s" % err)
            self.errorlogger.logger.error(
                "linear get feature importance error:\n %s" % traceback.format_exc())
            return None

    def predict_proba(self, x):
        """Calculate data predict probability value.

        Parameters
        --------
        x: pandas.DataFrame of shape (n_sample, n_features)
            test data

        Returns
        --------
        :return: predict probability value
        """
        return self._model.predict(x)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from .base_model import SklearnModel
from sklearn import svm
import time
import traceback
from ..ml_utils import runstatus


class LinerSVR(SklearnModel):
    """A liner classifier.
        Encapsulation form sklearn model.

    Parameters
    ----------
    config: the instance of ConfigParser.ConfigParser().

    Examples
    --------
        >>> from xes_ml_arch.src.model import linear
        >>> import configparser
        >>> import pandas as pd
        >>> import numpy as np
        >>> import random
        >>> config = configparser.ConfigParser()
        >>> config.read("liner.conf")
        >>> x = pd.read_csv('data.txt', sep=',')
        >>> y = np.array([int(random.random() * 100) for _x in range(99)])
        >>> ins = linear.Liner(config=config)
        >>> ins.init()
        >>> ins.train(x, y)
        ...
        ...
    """

    def __init__(self, config=None, log_path = None):
        super(LinerSVR, self).__init__(config=config, log_path = log_path)
        self._n_estimators = 200
        self._max_feature = "auto"
        self._max_deepth = None

    def _init_model(self):
        """Init model.

        Returns
        --------
        Boolean: True while suceess else False.
        """
        try:
            self._init_config()
            self._model = svm.LinearSVR(**self._model_params)
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("init linear model error: %s" % err)
            self.errorlogger.logger.error("init linear model error:\n %s" % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def _init_config(self):
        pass

    def train(self, x, y):
        """Model train function.

        Parameters
        ----------
        x: input sample data.
        y: label.

        Returns
        ---------
        Bool: True(train successed) or False(train faild).
        """

        try:
            t_start = time.time()
            self.managerlogger.logger.info("start linear..")
            self._model.fit(x, y)
            self.managerlogger.logger.info("finished liner!")
            t_end = time.time()
            self.managerlogger.logger.info("liner train time: %s " % (t_end - t_start))
            return runstatus.RunStatus.SUCC
        except Exception as err:
            self.managerlogger.logger.error("linear train error: %s " % err)
            self.errorlogger.logger.error("linear train error:\n %s " % traceback.format_exc())
            return runstatus.RunStatus.FAILED

    def get_feature_importance(self, feature):
        """Get feature weights of user data.

        Parameters
        ----------
        feature: Target characteristics(string of list).

        Returns
        ---------
        if faild return None,else return the score of feature importrance.(list, [[feature, score]]).
        """

        try:
            res = zip(self._model.coef_, feature)
            return res
        except Exception as err:
            self.managerlogger.logger.error("linear get feature importance error: %s" % err)
            self.errorlogger.logger.error(
                "linear get feature importance error:\n %s" % traceback.format_exc())
            return None

    def predict_proba(self, x):
        """Calculate data predict probability value.

        Parameters
        ----------
        x: input sample data.

        Returns
        ---------
        list of probability value [[ ]]
        """
        return self._model.predict(x)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import configparser
import random
import pandas as pd
import numpy as np

sys.path.append("../../../../")

from xes_ml_arch.src.analysis import analysis
from xes_ml_arch.src.ml import prediction_ml
from xes_ml_arch.src.ml_utils import runstatus


class TestAnalysis(unittest.TestCase):
    def setUp(self):
        self.conf = configparser.ConfigParser()
        self.conf_try = configparser.ConfigParser()
        self.conf_file = "./config/ml.conf"
        self.conf.read(self.conf_file)
        self.conf_try.read('./config/ml_try.conf')
        self.columns = ["col%s" % (x) for x in range(10)]
        self.x = self.columns[:9]
        self.y = self.columns[9]
        self.get_data()
        self.ins = prediction_ml.PredictionML(conf=self.conf, xeasy_log_path = '../log/log.conf')
        self.ins_no = prediction_ml.PredictionML(conf=self.conf_file,xeasy_log_path = '../log/log.conf')
        self.ins_try = prediction_ml.PredictionML(conf=self.conf_try,xeasy_log_path = '../log/log.conf')

    def test_set_data(self):
        self.assertTrue(self.ins.set_data(self._data))
        self.assertFalse(self.ins.set_data(self._data_no))
        self.ins_try.set_data(self._data)

    def test_start(self):
        self.ins.set_data(self._data)
        if self.ins.init() == runstatus.RunStatus.SUCC:
            self.ins.start()
        if self.ins_no.init() == runstatus.RunStatus.SUCC:
            self.ins_no.start()
        if self.ins_try.init() == runstatus.RunStatus.SUCC:
            self.ins_try.start()

    def test_init(self):
        assert (self.ins.init(), runstatus.RunStatus.SUCC)

    def get_data(self):
        self._data = pd.DataFrame(
            [[int(random.random() * 100) for x in range(10)] for y in range(10000)],
            columns=self.columns)
        self._data["col9"] = np.random.choice([0, 1], size=10000)
        self._data_no = np.random.randint(0, 5, 99)


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import traceback
from . import base_ml
import configparser
from ..constance.predict_constance import PredictConstance
from ..ml_utils import runstatus, global_pre
import os


class TrainML(base_ml.BaseML):
    """
    This basic class encapsulates the training of the model.

    Parameters
    --------
    conf : configparser.ConfigParser
        Model configuration file.

    Examples
    --------
    >>> from xes_ml_arch.src.ml import train_model_ml
    >>> from xes_ml_arch.src.ml_utils import global_pre
    >>> import configparser
    >>> import pandas as pd
    >>> BASE_CONFIG = "base_config"
    >>> RESULT_PATH = "result_path"
    >>> conf = configparser.ConfigParser()
    >>> ml_config = configparser.ConfigParser()
    >>> ml_config.read("./config/demo/ml_online.conf")
    >>> global_pre.RES_PATH = ml_config.get(BASE_CONFIG, RESULT_PATH)
    >>> ml = train_model_ml.TrainML(conf=ml_config)
    >>> ml.init()
    >>> ml.set_data(pd.read_csv("./data/sample.txt"))
    >>> ml.start()

    """

    def __init__(self, conf=None,xeasy_log_path = None):
        super(TrainML, self).__init__(config=conf,xeasy_log_path = xeasy_log_path)

    def start(self):
        """
        Start to train a model.

        Returns
        --------
        :return: bool, runstatus.RunStatus
            runstatus.RunStatus.SUCC: success
            runstatus.RunStatus.FAILD: faild
        """
        self.managerlogger.logger.info("start ml train model...")
        if not self._train_handle(None):
            # self.managerlogger.logger.error("train faild")
            return runstatus.RunStatus.FAILED
        self.managerlogger.logger.info("train succeed")

        if not self._get_result():
            # self.managerlogger.logger.error("get result faild")
            return runstatus.RunStatus.FAILED
        self.managerlogger.logger.info("get result succeed")

        if not self._store_predict_result():
            # self.managerlogger.logger.error("store result faild")
            return runstatus.RunStatus.FAILED
        self.managerlogger.logger.info("store result succeed")
        self.managerlogger.logger.info("finished ml train model!")
        return runstatus.RunStatus.SUCC

    def _train_handle(self, data):
        """
        Model train.

        Parameters
        --------
        data: pandas.DataFrame
            sample data

        Returns
        --------
        :return: bool
            True: success
            False: failed
        """
        # split data
        if not self._init_data_spliter():
            return False

        # set data and feature processor
        self._feature_processor.train_data = self._data_spliter.train_data
        self._feature_processor.test_data = self._data_spliter.test_data
        if not self._feature_processor.execute():
            return False

        # analysis data
        self._analysiser.reset(data=self._feature_processor.train_data,
                               feature_columns=self._feature_processor.data_feature_columns,
                               label_columns=self._feature_processor.data_target_column)
        if not self._analysiser.execute():
            return False

        # search for the best params
        if self._optimizing._enable_gridsearch == True:
            opt = self._optimizing.excute(self._model, self._model._model_params,
                                          self._feature_processor.train_data_feature,
                                          self._feature_processor.train_data_target)
            if not opt:
                return False
            self._model._model = self._optimizing.best_estimator_

        if runstatus.RunStatus.SUCC == self._model.train(self._feature_processor.train_data_feature,
                                                         self._feature_processor.train_data_target):
            self._model.store_model()

            # when model is dst get bintree
            try:
                tmp_file = os.path.join(global_pre.RES_PATH, "model_dest")
                self._model.show_dot(tmp_file, self._feature_processor.train_data_feature.columns)
            except:
                pass

            self._model.store_feature_importance(self._feature_processor.train_data_feature.columns)
        else:
            self.managerlogger.logger.error("model train faild")
            return False
        # Cross-validation
        self._cv_model()

        return True

    def _init_data_spliter(self):
        """
        Initialize the data set splitter.

        Returns
        --------
        :return: bool
            True: success
            False: faild
        """
        try:
            split_conf = self._conf.get(PredictConstance.BASE_CONFIG,
                                        PredictConstance.DATA_SPLITE_CONFIG)
            conf = configparser.ConfigParser()
            conf.read(split_conf)
            self._data_spliter.reset(conf=conf, data=self._data)
            if self._data_spliter.execute() == runstatus.RunStatus.SUCC:
                self.managerlogger.logger.info("init data spliter successful!")
                return True
        except Exception as err:
            self.managerlogger.logger.error("init data spliter error: %s" % err)
            self.errorlogger.logger.error("init data spliter error:\n %s" % traceback.format_exc())
        return False

    def _cv_model(self):
        """
        Cross-validate the trained model.

        Returns
        --------
        :return: bool
            True: success
            False: fialed
        """
        self._cross_validation.reset(data=self._feature_processor.train_data,
                                     x_columns=self._feature_processor.data_feature_columns,
                                     y_column=self._feature_processor.data_target_column,
                                     id_fields=self._feature_processor.data_id_columns)

        try:
            # model_config = self._conf.get(PredictConstance.BASE_CONFIG,
            #                               PredictConstance.MODEL_CONFIG)
            # conf = ConfigParser.ConfigParser()
            # conf.read(model_config)
            # cv_model = model_factory.ModelFactory.create_model(config=conf)
            cv_model = self._model
            if self._cross_validation.execute() == runstatus.RunStatus.SUCC:
                if self._cross_validation.cv_model(cv_model):
                    self.managerlogger.logger.info("cv success")
            return True
        except Exception as err:
            self.managerlogger.logger.error("init model error: %s" % err)
            self.errorlogger.logger.error("init model error: \n %s" % traceback.format_exc())
            return False



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

from . import syslog


class SysManagerLog(syslog.SysLog):
    def __init__(self, filename, log_path = None):
        """Instantiate the system manager log class"""
        syslog.SysLog.__init__(self, filename, name='system', path = log_path)



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT


import configparser
import unittest
import sys
import pandas as pd

sys.path.append("../../../../")
from xes_ml_arch.src.feature_enginnering import xes_onehot_encoder


class TestOneHot(unittest.TestCase):
    def setUp(self):
        self._data_path = "../data/test_onehot.txt"
        self._config = "./conf/test_onehot.conf"
        self._conf = configparser.ConfigParser()
        self._conf.read(self._config)
        self._config_no = "./conf/test_onehot_no.conf"
        # self._conf_no = configparser.ConfigParser()
        # self._conf_no.read(self._config_no)

        self._data = pd.read_csv(self._data_path)
        # self._data_no = pd.read_csv("../data/test_oneshot_none.txt")
        self._onehot_ins = xes_onehot_encoder.XESOneHotEncoder(data=self._data, conf=self._conf,log_path = '../log/log.conf')
        self._onehot_ins_no = xes_onehot_encoder.XESOneHotEncoder(data=pd.DataFrame(), conf=self._conf,log_path = '../log/log.conf')
        self._onehot_ins_ee = xes_onehot_encoder.XESOneHotEncoder(data=self._data, conf='',log_path = '../log/log.conf')
        self._onehot_ins_nn = xes_onehot_encoder.XESOneHotEncoder(data=self._data, conf=self._config_no,log_path = '../log/log.conf')

    def test_start(self):
        self.assertTrue(self._onehot_ins.init())
        self.assertTrue(self._onehot_ins.execute())
        self.assertFalse(self._onehot_ins_no.execute())
        self.assertFalse(self._onehot_ins_ee.execute())
        self.assertFalse(self._onehot_ins_nn.execute())

    def test_get_data(self):
        self._onehot_ins.reset(data=self._data, one_hot_file=self._conf)
        self._onehot_ins.execute()
        data = self._onehot_ins.get_data

    def test_reset(self):
        data_no = pd.DataFrame()
        self.assertFalse(self._onehot_ins.reset(data=data_no, one_hot_file=''))
        self.assertFalse(self._onehot_ins.reset(data=self._data, one_hot_file=''))


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import random
import pandas as pd

sys.path.append('../../../..')
from xes_ml_arch.src.ml_utils import feature_processor


class TestUtils(unittest.TestCase):
    def setUp(self):
        self.covertint = 3
        self.covertflo = 3.3
        self.covertstr = '3.3'
        self.covertbool = (6,7)

        self.time1 = pd.Series(pd.date_range(start='2019-1-09',periods=10,freq='H'))
        self.time2 = pd.Series(pd.date_range(start='2017-2-21',periods=10,freq='H'))
        self.stamp = pd.Series(feature_processor.FeatureProcessor().time2stamp(self.time2))

        self.year = ['2020+','2031+']
        self.age = ['28-30','28+3']

        self.disli = [random.randint(1,15) for _ in range(100)]
        self.day = 100000

    def test_cover2int(self):
        resa = feature_processor.FeatureProcessor().convert_to_int(self.covertint)
        resb = feature_processor.FeatureProcessor().convert_to_int(self.covertflo)
        resc = feature_processor.FeatureProcessor().convert_to_int(self.covertstr)
        self.assertTrue(isinstance(resa, int))
        self.assertTrue(isinstance(resb, int))
        self.assertEqual(resc,0)

    def test_cover2str(self):
        #resa = feature_processor.FeatureProcessor().convert_to_str(self.covertbool)
        resb = feature_processor.FeatureProcessor().convert_to_str(self.covertflo)
        resc = feature_processor.FeatureProcessor().convert_to_str(self.covertstr)
        self.assertTrue(isinstance(resc, str))
        self.assertTrue(isinstance(resb, str))
        #self.assertEqual(resa, '')

    def test_time2stamp(self):
        resa = feature_processor.FeatureProcessor().time2stamp(self.time1)
        self.assertIsNotNone(resa)

    def test_stamp2time(self):
        resa = feature_processor.FeatureProcessor().stamp2time(self.stamp)
        self.assertIsNotNone(resa)

    def test_timediff(self):
        _diff = feature_processor.FeatureProcessor().time_diff(self.time1,self.time2)
        self.assertIsNotNone(_diff)

    def test_minusdata(self):
        _res = feature_processor.FeatureProcessor().minus_data(self.time1, self.time2)
        self.assertIsNotNone(_res)

    def test_absminusdata(self):
        _res = feature_processor.FeatureProcessor().abs_minus_data(self.time1, self.time2)
        self.assertIsNotNone(_res)

    def test_stayinyear(self):
        res = feature_processor.FeatureProcessor().stay_in_year(self.year)
        self.assertIsNotNone(res)

    def test_age2int(self):
        res = feature_processor.FeatureProcessor().age2int(self.age)
        self.assertNotEqual(res,[-1 for _ in range(len(self.age))])

    def test_discretizefreque(self):
        res = feature_processor.FeatureProcessor().discretize_freque(self.disli)
        self.assertIsNotNone(res)
    def test_getdayrange(self):
        ans = res = feature_processor.FeatureProcessor().get_day_range(self.day)
        self.assertTrue(res > 0)


if __name__ == '__main__':
    unittest.main()



# -*-coding:utf-8-*-
# @version: 0.0.1
# License: MIT

import unittest
import sys
import pandas as pd
import configparser
import numpy as np

sys.path.append("../../../..")

from xes_ml_arch.src.feature_enginnering import data_processor


class TestDataProcessor(unittest.TestCase):
    def setUp(self):
        self._data_path = "../data/test.txt"
        self._config = "./conf/feature_enginnering.conf"
        self._data = pd.read_csv(self._data_path)
        self._conf = configparser.ConfigParser()
        self._conf_no = configparser.ConfigParser()
        self._conf_false = configparser.ConfigParser()
        self._conf.read(self._config)
        self._conf_no.read("./conf/feature_enginnering_no.conf")
        self._conf_false.read("./conf/feature_enginnering_false.conf")
        self.target = np.random.randint(0, 4, 100)
        self.feature_processor = data_processor.DataProcessor(conf=self._conf, log_path = '../log/log.conf')
        self.feature_processor_none = data_processor.DataProcessor(conf=self._conf, log_path = '../log/log.conf')
        self.feature_processor_no = data_processor.DataProcessor(conf=self._conf_no, log_path = '../log/log.conf')

    def test_start(self):
        self.assertTrue(self.feature_processor.init())
        self.feature_processor.train_data = self._data
        self.feature_processor.test_data = self._data
        self.assertTrue(self.feature_processor.execute())

    def test_start_false(self):
        self.assertTrue(self.feature_processor.init())
        self.assertFalse(self.feature_processor.execute())
        self.assertFalse(self.feature_processor_no.init())
        self.assertFalse(self.feature_processor.execute())
        self.assertTrue(self.feature_processor_none.init())
        self.assertFalse(self.feature_processor.execute())

    # def test_set_data(self):
    #     self.assertTrue(self.feature_processor.init())
    #     self.assertTrue(self.feature_processor.set_data(self._data))
    #     data_no = None
    #     self.assertFalse(self.feature_processor.set_data(data_no))


if __name__ == '__main__':
    unittest.main()
